# -*- coding: utf-8 -*-
"""
Created on Thu Nov 15 16:56:01 2012

@author: jamesbono

Here's a test script for use in developing the PyNFG library
"""

import pdb
import pymc as pm
import numpy as np
from types import *
from collections import OrderedDict
from det_chance_node import DetChanceNode
from stc_chance_node import StcChanceNode

a = Decision_Node(name='a', space=[0,1], player='nature', \
                  description='parent node of b and d')
c = Decision_Node(name='c', player='nature', \
                  description='parent node of b and d', space=[0,1])
d = DetChanceNode(name='d', space =[0,1], \ 
                  description='child node of node a and c') #parents will be captured from the eval_func
e = StcChanceNode(name='e', space =[-2,-1,0,1], \
                  description='child node of node a and c', 
                  builtin_dist="DiscreteUniform", bias_lower = -2.0, upper=c)   
b = Decision_Node(name='b', player='James', \
                  description='child node of d and e', space=[0,1], 
                  parents={'e': e, 'd': d})
                    
# Initialize empty CPT's for each node                    
a.initialize_CPT()
c.initialize_CPT()
b.initialize_CPT()

# Fill the CPTs with probabilities
a.CPT = np.array([.25, .75])
c.CPT = np.array([.25, .75])
b.CPT = np.array([[[.25, .75], 
                   [.65, .35],
                   [.35, .65],
                   [.45, .55]],
                    
                  [[.65, .35],
                   [.75, .25],
                   [.55, .45],
                   [.85, .15]]])

# Convert the Decision_Node objects to pymc
a.convert_to_pymc()
c.convert_to_pymc()
    # Define the eval_func for the deterministic nodes and convert the nodes to pymc objects
def copy_the_parent(parent1 = a.pymc, parent2 = c.pymc):
    return (parent1 + parent2)%2
d.eval_func = copy_the_parent
d.convert_to_pymc()    
    
# Convert the stochastic node and final decision node to pymc object
e.convert_to_pymc()
b.convert_to_pymc()

# testing out the random functions
print 'Taking random draws gives:'
print "a = ", a.pymc.random()
print "b = ", b.pymc.random()
print "c = ", c.pymc.random()
print "d = ", d.pymc.value
print "e = ", e.pymc.random()

# testing the deterministic node output when it's eval function changes on the fly
copy_the_parent = lambda parent1 = a.pymc, parent2 = c.pymc: \
    (parent1*2 - parent2)%2
d.eval_func = copy_the_parent
d.convert_to_pymc()
print "d new value = ", d.pymc.value

# testing out the logp functions
print 'The log probabilities are:'
print "a_logp = ", a.pymc.logp
print "c_logp = ", c.pymc.logp
print "b_logp = ", b.pymc.logp
print "e_logp = ", e.pymc.logp

# create a dictionary of the pymc objects
node_dict = {'a_pymc': a.pymc, 'c_pymc': c.pymc, 'b_pymc': b.pymc, \
             'd_pymc': d.pymc, 'e_pymc': e.pymc }
M = pm.MCMC( node_dict )
M.sample( 110, burn=10, verbose=2 )

# plotting the samples
#    pm.Matplot.plot( M )
#    x = gcf()
#    x.savefig( 'plotb', format='png' )