function [defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM, ...
          defAvgNumMove, defAvgNumMoveSEM, AvgSocWelf, AvgSocWelfSEM ] ... 
                = Average_rewards( Defender, Attacker, N, parallel, ppack )
%Compute the players' average rewards over N episodes
%Inputs:    Defender = defender
%           Attacker = attacker
%           N = number of episodes to average
%           parallel = set to 'true' to use parallel computing toolbox
%global params;

%Make sure number of samples is positive
assert( N > 0 );

meanRD = zeros(1,N);
meanRA = zeros(1,N);

tic     %start timer
%Use parallel computing toolbox if true
if ( parallel )
%     distcomp.feature( 'LocalUseMpiexec', false );
%     matlabpool open;
%     parfor i = 1:N
%         i;
%         Attacker.exists = genAttExists(params.ProbAttExist); %pick attacker mode based on the probability of attacker existence
%         globalBridge( params );
%         %calculate the rewards for each episode
%         [~, defRewards, attRewards] = Simulate( Defender, Attacker, false,params );
%         %average over each episode
%         meanRD(i) = mean(defRewards);
%         meanRA(i) = mean(attRewards);
%     end
%     matlabpool close;
%otherwise just use regular for loop
else
    for i=1:N
        %i
        %calculate the rewards for each episode
        Attacker.exist = genAttExists(ppack.params.ProbAttExist);
                %pick attacker mode based on the probability of attacker existence
        [~, defRewards, attRewards, ~, ~, ~, NumDefMove] = Simulate( Defender, Attacker, false, ppack );
        %total rewards over each episode
        meanRD(i) = sum(defRewards)/ppack.params.nSimSteps;
        meanRA(i) = sum(attRewards)/ppack.params.nSimSteps;
        defMovesNum(i) = NumDefMove;
        SocWelf(i) = meanRD(i) - defMovesNum(i);
    end
end
toc     %stop timer

%average over N episodes
defAvgReward = mean(meanRD);
defAvgRewardSEM = sqrt(var(meanRD)/N);
attAvgReward = mean(meanRA);
attAvgRewardSEM = sqrt(var(meanRA)/N);
defAvgNumMove = mean(defMovesNum);
defAvgNumMoveSEM = sqrt(var(defMovesNum)/N);
AvgSocWelf = mean(SocWelf);
AvgSocWelfSEM = sqrt(var(SocWelf)/N);

%normalization of the rewards
if(0) %if you want to normalize the rewards
    meanRD = meanRD/max(abs(ppack.ER.Def(:)));
    %meanRD = (meanRD+5)/5; 
    meanRA = meanRA/max(abs(ppack.ER.Att(:)));
    defMovesNum = defMovesNum/ppack.params.nSimSteps;
    %defMovesNum = (ppack.params.nSimSteps-defMovesNum)/ppack.params.nSimSteps;
    for i=1:N
        SocWelf(i) = meanRD(i) - defMovesNum(i);
    end
    defAvgReward = mean(meanRD);
    defAvgRewardSEM = sqrt(var(meanRD)/N);
    attAvgReward = mean(meanRA);
    attAvgRewardSEM = sqrt(var(meanRA)/N);
    defAvgNumMove = mean(defMovesNum);
    defAvgNumMoveSEM = sqrt(var(defMovesNum)/N);
    AvgSocWelf = mean(SocWelf);
    AvgSocWelfSEM = sqrt(var(SocWelf)/N); 
end



end