function A = Attacker_policy(Attacker, MA, ppack)
% Simulate the attacker's policy (continuous version)
% Inputs:    Attacker = attacker
%            OA = attacker's observation
%            MA = attacker's memory
%            global params
%            Bins of q3
% Output:    q3 = Attacker's move in index

params = ppack.params;
BinsOrSet = ppack.BinsOrSet;

if Attacker.exist == 0 % does not exist
    q3 = 0;
    
else % exist
    % Fixed move
    if ( Attacker.mode == 1 )
        q3 = Attacker.thetaA;
    % Randomize uniformly
    elseif ( Attacker.mode == 2 )
        mnew = randi([-params.M,params.M],1);
        q3 = mnew*params.p3max/params.M;
    % Scott's policy
    elseif (Attacker.mode == 3)
        m_prev = MA.q3_prev * params.M / params.p3max;  % invert equation to find m_prev
        [mstar,Vstar] = Find_star( MA.V2, m_prev, params );
        if ( Vstar > Attacker.thetaA )
            mnew = mstar;
        else
            mnew = m_prev - sign( MA.V2 - params.V0 );
        end
        q3 = mnew*params.p3max/params.M;
    % Stochastic CPT policy
    elseif (Attacker.mode == 4)
        % To be ported...
    % Based on linear fitted Q iteration. Qwts
    elseif ( Attacker.mode == 5 )
        S = [MA.V2; MA.V3; MA.A_prev.q3; MA.SA];
        [~,q3i] = FitQLin_GetQmax( Attacker.QWts, S, ppack.BinsOrSet.q3Set);
        q3 = ppack.BinsOrSet.q3Set(q3i);
    end
end

A.p3 = params.p3;
A.q3 = q3;

end

function [mstar,Vstar] = Find_star( V2, m_prev, params )
%Find the star quantities as in Scott's writeup

q3max = params.p3max;

m = [-params.M:params.M]';
arg = abs(V2+(m-m_prev)*q3max*params.x2/params.M/params.V0-params.V0);
[~,ind] = max(arg);
mstar = m(ind);
Vstar = abs(max(arg));

end