function dX = Defender_L0( Defender, MD, params )
%Simulate the defender's policy
%Inputs:    Defender = defender
%           MD = defender's memory
%Output:    dX = Defender's move
%global params;

%Choose the policy to play according to Defender.mode
%Parameters to the policy are generally in Defender.thetaD

%Fixed move
if( Defender.mode == 1 )
    dX = Defender.thetaD; 
    assert( dX == -1 || dX == 0 || dX == 1);
%Randomize uniformly between {-1,0,1}
elseif( Defender.mode == 2 )
    dX = randi([-1,1],1);
%According to policy in Sec 2.5 in writeup
elseif( Defender.mode == 3 )
    x = round((MD.V1index-params.V0)*params.N/params.dV);   %invert equation to get x from V1
    V2 = MD.V2;
    V3 = MD.V3;

    z=((V2+V3)/2-params.V0)/(params.dV/params.N);
    PiD_normal =  -sign(z) * sign( floor( abs(z) ));
    PiD_safe = -sign(x);
    
    dX = sign((1-Defender.thetaD)*PiD_normal + Defender.thetaD*PiD_safe);
end

end
