function dX = Defender_policy(Defender, MD, params)
% Simulate the defender's policy (continuous version)
% Inputs:    Defender = defender
%            MD = defender's memory
%            CPTs
%            size of the set of dX
%            Global parameters
% Output:    dX = Defender's move

    % Choose the policy to play according to Defender.mode
    % Parameters to the policy are generally in Defender.thetaD

    % Fixed move
    if ( Defender.mode == 1 )
        dX = Defender.thetaD;
        assert( dXnew == -1 || dXnew == 0 || dXnew == 1);
    % Randomize uniformly in the dX set
    elseif ( Defender.mode == 2 )
        dX = randi([-1,1],1);
    % Scott's Defender's L0 policy
    elseif (Defender.mode == 3)        
        x = round((MD.V1-params.V0)*params.N/params.dV);   %invert equation to get x from V1
        z = ((MD.V2+MD.V3)/2-params.V0)/(params.dV/params.N);
        PiD_normal =  -sign(z) * sign( floor( abs(z) ));
        PiD_safe = -sign(x);
        dX = sign((1-Defender.thetaD)*PiD_normal + Defender.thetaD*PiD_safe);
    % Neural network policy
    elseif ( Defender.mode == 4 )
        % To be ported...
    % Based on linear fitted Q iteration. Qwts
    elseif ( Defender.mode == 5 )
        S = [MD.V1; MD.V2; MD.V3; MD.D_prev; MD.SD];
        [~,dxi] = FitQLin_GetQmax( Defender.QWts, S, params.dXset);
        dX = params.dXset(dxi);
    end
end
