function [states, defRewards, attRewards, Av_defRewards, Av_attRewards, switchtime, NumDefMove] = Simulate( Defender, Attacker, makePlot, ppack, entryTime )
%Simulate an episode
%Inputs:    Defender = defender
%           Attacker = attacker
%           makeplot = set to 'true' to display plots

%fprintf('\nContinuous environment simulation');
if ( ppack.params.player_type == 1 )
%    fprintf('\nContinuous players');
elseif ( ppack.params.player_type == 2 )
%    fprintf('\nDiscrete players');
end

if( ~exist('entryTime'))
    entryTime = inf;
end

%Preallocate tracking variables
defRewards = zeros(ppack.params.nSimSteps,1);
attRewards = zeros(ppack.params.nSimSteps,1);

defrew_av=0;   % average reward for the defender
attrew_av=0;   % average reward for the attacker
Av_defRewards = zeros(1,ppack.params.nSimSteps);
Av_attRewards = zeros(1,ppack.params.nSimSteps);

Av_attRewards_old = 0.5;

B = [];

if (0) %if learning defender is simulated
    s_i=1;
    d_pi2 = Defender.pi;  %L2Defender CPT
    d_pi1 = Defender.L1Defender.pi; %L1Defender CPT
    Attacker.mode = 3; %start with Scott's L0 attacker
end

NumDefMove = 0;
OldDefMove = 1;
for t = 1:ppack.params.nSimSteps
    
    if (t >= entryTime)       % introduce attacker in middle of simulation
        Attacker.exist = 1;
    end
    
    B = Sample_slice(B, Defender, Attacker,ppack); %Take a timestep
    
    %Save to output
    states{t} = B;
    defRewards(t) = Defender_reward(B, ppack.params );
    attRewards(t) = Attacker_reward(B, ppack.params );
    
    defrew_av=defrew_av+(defRewards(t)-defrew_av)/t;
    attrew_av=attrew_av+(attRewards(t)-attrew_av)/t;
        
    Av_defRewards(t) = defrew_av;
    Av_attRewards(t) = attrew_av;
    
    if B{6}~=0
        NumDefMove = NumDefMove + 1;
    end
    OldDefMove = B{6};
    
    if(0) %if a learning L2Defender is simulated
        if (mod(t,10) == 0)
            if Av_attRewards(t) > Av_attRewards_old
                if Defender.pi == d_pi1
                    Defender.pi = d_pi2;
                else
                    Defender.pi = d_pi1;
                end
                t
                switchtime(s_i) = t;
                s_i=s_i+1;
            end
            Av_attRewards_old = Av_attRewards(t);
        end
        if t==60
            Attacker.mode = ppack.params.policy_mode; %switch to an L1 attacker
        end
    end

end
%Set flag to plot
if ( makePlot )
  %  Plot the various variables over time
    Plot_S(states);
    Plot_MD(states);
    Plot_MA(states);
    Plot_Defender(states,ppack.params);
    Plot_Attacker(states,ppack);
    Plot_RD(defRewards,Av_defRewards);
    Plot_RA(attRewards,Av_attRewards);
    Plot_L(states);
    
end
switchtime = 1; %this is a random assignment to prevent errors
end