function BinsOrSet = Save_BinsOrSet(params)
% This function is used to compute and save the bins or the 
%      set of possible values for various nodes
% Input: global parameters
% Output: BinsOrSet

    % The number of possible x
    BinsOrSet.N_x = (params.N - (-params.N)) + 1; % x varies between -N and N
    % The set of possible x
    BinsOrSet.xSet = linspace(-params.N, params.N, BinsOrSet.N_x);
    % The number of possible V1
    BinsOrSet.N_V1 = BinsOrSet.N_x; % V1 and x have 1-to-1 relationship
    % The set of possible V1
    BinsOrSet.V1set = params.V0 + BinsOrSet.xSet * params.dV / params.N;
    
    % V2 and V3 bin boundaries.
    if ( params.V2n3binConfig == 1 )
        BinsOrSet.V2n3bins = [-inf, 0.95, 0.97, 1.03, 1.05, inf];
    elseif ( params.V2n3binConfig == 2 )
        BinsOrSet.V2n3bins = [-inf, linspace(0.94,1.06,7), inf];
    else
        assert(false);
    end
        
    % The number of bins for V2 and V3
    [~,TemSize] = size(BinsOrSet.V2n3bins); % Number of elements in the array of V2n3bins
    BinsOrSet.N_V2 = TemSize - 1; % Number of bins for V2
    BinsOrSet.N_V3 = TemSize - 1; % Number of bins for V3
    
    % The number of possible defender's action dX
    [~,BinsOrSet.N_dX] = size(params.dXset);
    
    % The possible moves by the attacker
    BinsOrSet.mSet = -params.M:params.M; % m can vary from -M to M
    % The number of possible m
    [~,BinsOrSet.N_m] = size(BinsOrSet.mSet);
    % The number of bins for q3
    BinsOrSet.N_q3 = BinsOrSet.N_m; %q3 and m have a 1-on-1 relationship
    % The possible values for q3
    BinsOrSet.q3Set = BinsOrSet.mSet * params.p3max / params.M; 
    
    % The number of bins for p2
    BinsOrSet.N_p2 = round((params.p2max - params.p2min)/params.dp2); % p2 varies between p2min and p2max
    % The bins for p2
    BinsOrSet.p2bins = linspace(params.p2min, params.p2max, ...
        BinsOrSet.N_p2 + 1);
    
    % Bins for statisticD and statisticA
    if( params.mem.mode_Def == 1 )
        asym_D = 1/(1-params.mem.gamma_Def); % The positive asymptote value of statisticD
        BinsOrSet.statisticDbins = linspace(-asym_D,asym_D,params.N_statisticD + 1);
    elseif ( params.mem.mode_Def == 2 )
        BinsOrSet.statisticDbins = [-inf,1.5:params.N_statisticD-0.5,inf];
    end
    asym_A = 1/(1-params.mem.gamma_Att); % The postive asymptote value of statisticA
    BinsOrSet.statisticAbins = linspace(-asym_A,asym_A,params.N_statisticA + 1);
    
end
        
    
    
    
    
    
    
    
    