function [CPT_V1] = GenCPT_V1(params, BinsOrSet,T)
% This function is used to generate conditional probability
% table for V1.
% Input: Global parameters
%        the bins for V1 and the set for possible dX
% Outputs: CPT_V1 

    % Initialize CPT_V1. V1@t = V1(V1@t-1, dX)
    CPT_V1 = zeros(BinsOrSet.N_V1, BinsOrSet.N_V1, BinsOrSet.N_dX);
    
    if ( ~exist('T') )
        T = 10; % Default number of times we repeat for each combination of inputs
    end
    
    for j = 1:BinsOrSet.N_dX % for each possible defender's action dX
        dX = params.dXset(j); 
        for i = 1:BinsOrSet.N_V1 % for each possible value of V1
            V1_prev = BinsOrSet.V1set(i) * ones(T,1); % Repeat T times
            
            Num = zeros(BinsOrSet.N_V1,1); % Initialize the tracker
            
            x_prev = round((V1_prev - params.V0) * params.N / params.dV); % x@t-1. Reverse the V1 equation.
            x = x_prev + dX; % Update x based on defender's input
            x = min(x,params.N); % Saturator
            x = max(x,-params.N);
            
            V1 = params.V0 + x * params.dV / params.N; % V1@t
            [~,p] = histc(V1,BinsOrSet.V1set); % Map V1 back to an bin
            for s = 1:BinsOrSet.N_V1 % for each bin of V1
                Num(s,1) = sum(p==s); % Count the number that this bin is arrived
            end 
            CPT_V1(:,i,j) = Num(:,1)/T; % Probabilities given V1@t-1 and dX
        end
    end
end

    
                
                
                
                
                
                
                
                
                