function [CPT_V3] = GenCPT_V3(params, BinsOrSet,T)
% This function is used to generate the conditional probability table for
% V3
% Input: Global parameters
%        the bins of V1 and A
% Output: CPT_V3
    
    % Initialize CPT_V3. V3 = V3(V2,A).
    % p3 is fixed. q3 fully determines A.
    CPT_V3 = zeros(BinsOrSet.N_V3,BinsOrSet.N_V2, BinsOrSet.N_q3);
    
    if ( ~exist('T') )
        T = 10000; % Number of times we repeat for each combination of inputs
    end
    
    for j = 1:BinsOrSet.N_q3 % for each bin of q3
        q3 = BinsOrSet.q3Set(j) * ones(T,1);
        for i = 1:BinsOrSet.N_V2 % for each bin of the V2
            Num = zeros(BinsOrSet.N_V3,1); % Initialize the tracker
            % Generate V2 uniformly from the bin
            if (i == 1) % Apply half-Gussian distribution. Only sample the left to the mean 
                mu = BinsOrSet.V2n3bins(2); % mean = upperbound of the lowest bin
                sigma = (BinsOrSet.V2n3bins(3)-BinsOrSet.V2n3bins(2))/sqrt(1-2/pi); % Stdev = range of right neighboring bin divided by a difference factor
                Ran = mu + sigma .* randn(T,1); % Generate full-normally distributed numbers
                V2 = BinsOrSet.V2n3bins(2) - abs(BinsOrSet.V2n3bins(2) - Ran); % Convert full-normal to half-normal
            elseif (i == BinsOrSet.N_V2) % Apply half-Guassian distribution. Only sample the right to the mean. 
                mu = BinsOrSet.V2n3bins(i); % mean = lowerbound of the highest bin
                sigma = (BinsOrSet.V2n3bins(i) - BinsOrSet.V2n3bins(i-1))/sqrt(1-2/pi); % Stdev = range of the left neighboring bin divided by a difference factor
                Ran = mu + sigma .* ones(T,1); % Generate full-normally distributed numbers
                V2 = BinsOrSet.V2n3bins(i) + abs(BinsOrSet.V2n3bins(i) - Ran); % Convert full-normal to half-normal
            else % Sample uniformly in the medium three bins
                V2 = BinsOrSet.V2n3bins(i) + (BinsOrSet.V2n3bins(i+1) - ...
                      BinsOrSet.V2n3bins(i)) .* rand(T,1);
            end
            V3 = V2 - (params.r2*(-params.p3) + params.x2*(-q3))/params.V0; % Update V3 given inputs
            [~,p] = histc(V3,BinsOrSet.V2n3bins); % Map V3 back to an bin
            for s = 1:BinsOrSet.N_V3 % for each bin of V3
                Num(s,1) = sum(p==s); % Count the number that this bin is arrived
            end
            CPT_V3(:,i,j) = Num(:,1)/T; % Probabilities given V2 and A
        end
    end
end
                        
                        
                        
                        
                        