function [CPT_statisticA] = GenCPT_statisticA(params, BinsOrSet,T)
% This function is used to generate the CPT for statistic A
% Inputs: global params; BinsOrSet
% Output: CPT_statisticA

    % Initialize the output
    CPT_statisticA = zeros(params.N_statisticA,params.N_statisticA,...
        BinsOrSet.N_V3,BinsOrSet.N_V3,BinsOrSet.N_q3, BinsOrSet.N_q3);
    
    if ( ~exist('T') )
        T = 10000; % The number of repeat times
    end
    
    for n = 1:BinsOrSet.N_q3 % for each value of q3
        q3_2prev = BinsOrSet.q3Set(n) * ones(T,1); % q3 @ t-2
        
        for m = 1:BinsOrSet.N_q3 % for each value of q3
            q3_prev = BinsOrSet.q3Set(m) * ones(T,1); % q3 @ t-1
            
            for k = 1:BinsOrSet.N_V3 % for each bin of V3
                % Generate V3 @ t-2 uniformly from the bin
                if (k == 1) % Apply half-Gussian distribution. Only sample the left to the mean 
                    mu = BinsOrSet.V2n3bins(2); % mean = upperbound of the lowest bin
                    sigma = (BinsOrSet.V2n3bins(3)-BinsOrSet.V2n3bins(2))/sqrt(1-2/pi); % Stdev = range of right neighboring bin divided by a difference factor
                    Ran = mu + sigma .* randn(T,1); % Generate full-normally distributed numbers
                    V3_2prev = BinsOrSet.V2n3bins(2) - abs(BinsOrSet.V2n3bins(2) - Ran); % Convert full-normal to half-normal
                elseif (k == BinsOrSet.N_V3) % Apply half-Guassian distribution. Only sample the right to the mean. 
                    mu = BinsOrSet.V2n3bins(k); % mean = lowerbound of the highest bin
                    sigma = (BinsOrSet.V2n3bins(k) - BinsOrSet.V2n3bins(k-1))/sqrt(1-2/pi); % Stdev = range of the left neighboring bin divided by a difference factor
                    Ran = mu + sigma .* randn(T,1); % Generate full-normally distributed numbers
                    V3_2prev = BinsOrSet.V2n3bins(k) + abs(BinsOrSet.V2n3bins(k) - Ran); % Convert full-normal to half-normal
                else % Sample uniformly in the medium three bins
                    V3_2prev = BinsOrSet.V2n3bins(k) + (BinsOrSet.V2n3bins(k+1)...
                        - BinsOrSet.V2n3bins(k)) .* rand(T,1);
                end
                
                for j = 1:BinsOrSet.N_V3 % for each bin of V3
                % Generate V3 @ t-1 uniformly from the bin
                    if (j == 1) % Apply half-Gussian distribution. Only sample the left to the mean 
                        mu = BinsOrSet.V2n3bins(2); % mean = upperbound of the lowest bin
                        sigma = (BinsOrSet.V2n3bins(3)-BinsOrSet.V2n3bins(2))/sqrt(1-2/pi); % Stdev = range of right neighboring bin divided by a difference factor
                        Ran = mu + sigma .* randn(T,1); % Generate full-normally distributed numbers
                        V3_prev = BinsOrSet.V2n3bins(2) - abs(BinsOrSet.V2n3bins(2) - Ran); % Convert full-normal to half-normal
                    elseif (j == BinsOrSet.N_V3) % Apply half-Guassian distribution. Only sample the right to the mean. 
                        mu = BinsOrSet.V2n3bins(j); % mean = lowerbound of the highest bin
                        sigma = (BinsOrSet.V2n3bins(j) - BinsOrSet.V2n3bins(j-1))/sqrt(1-2/pi); % Stdev = range of the left neighboring bin divided by a difference factor
                        Ran = mu + sigma .* randn(T,1); % Generate full-normally distributed numbers
                        V3_prev = BinsOrSet.V2n3bins(j) + abs(BinsOrSet.V2n3bins(j) - Ran); % Convert full-normal to half-normal
                    else % Sample uniformly in the medium three bins
                        V3_prev = BinsOrSet.V2n3bins(j) + (BinsOrSet.V2n3bins(j+1)...
                            - BinsOrSet.V2n3bins(j)) .* rand(T,1);
                    end
                    
                    for i = 1:params.N_statisticA % for each bin
                        statisticA_prev = BinsOrSet.statisticAbins(i)+...
                            (BinsOrSet.statisticAbins(i+1) - ...
                             BinsOrSet.statisticAbins(i)) * rand(T,1); % Uniformly sample in the bin
                         
                        Num = zeros(params.N_statisticA, 1); % Initialize the tracker
                        
                        dV3 = V3_prev - V3_2prev;
                        dq3 = q3_prev - q3_2prev;
                        
                        statisticA = params.mem.gamma_Att * statisticA_prev...
                            + sign(floor((dV3-dq3*params.x2/params.V0)/...
                            (params.dV/params.V0)));
                        
                        [Num,p] = histc(statisticA, BinsOrSet.statisticAbins); % Map back to a binz
                        CPT_statisticA(:,i,j,k,m,n) = Num(1:end-1,1)/T; % Conditional probabilities
                    end
                end
            end
        end
    end
end
                         
                            
                        
                        
                        
                        
                        