function [CPT_statisticD] = GenCPT_statisticD(params,BinsOrSet,T)
% This function is used to generate the CPT for statistic D
% Inputs: global parameters; Bins for V1, V3 and statisticD
% Output: CPT_statisticD

    % Initialize the output
    CPT_statisticD = zeros(params.N_statisticD, params.N_statisticD,...
                      BinsOrSet.N_V1, BinsOrSet.N_V1, BinsOrSet.N_V3,...
                      BinsOrSet.N_V3);
                  
    if ( ~exist('T') )
        T = 10000; % The number of repeat times
    end
    
    for n = 1:BinsOrSet.N_V3 % for each bin of V3
        % Generate V3 uniformly from the bin
        if (n == 1) % Apply half-Gussian distribution. Only sample the left to the mean 
            mu = BinsOrSet.V2n3bins(2); % mean = upperbound of the lowest bin
            sigma = (BinsOrSet.V2n3bins(3)-BinsOrSet.V2n3bins(2))/sqrt(1-2/pi); % Stdev = range of right neighboring bin divided by a difference factor
            Ran = mu + sigma .* randn(T,1); % Generate full-normally distributed numbers
            V3_prev = BinsOrSet.V2n3bins(2) - abs(BinsOrSet.V2n3bins(2) - Ran); % Convert full-normal to half-normal
        elseif (n == BinsOrSet.N_V3) % Apply half-Guassian distribution. Only sample the right to the mean. 
            mu = BinsOrSet.V2n3bins(n); % mean = lowerbound of the highest bin
            sigma = (BinsOrSet.V2n3bins(n) - BinsOrSet.V2n3bins(n-1))/sqrt(1-2/pi); % Stdev = range of the left neighboring bin divided by a difference factor
            Ran = mu + sigma .* randn(T,1); % Generate full-normally distributed numbers
            V3_prev = BinsOrSet.V2n3bins(n) + abs(BinsOrSet.V2n3bins(n) - Ran); % Convert full-normal to half-normal
        else % Sample uniformly in the medium three bins
            V3_prev = BinsOrSet.V2n3bins(n) + (BinsOrSet.V2n3bins(n+1)...
                - BinsOrSet.V2n3bins(n)) .* rand(T,1);
        end
        
        for m = 1:BinsOrSet.N_V3 % for each bin of V3
            % Generate V3 uniformly from the bin
            if (m == 1) % Apply half-Gussian distribution. Only sample the left to the mean 
                mu = BinsOrSet.V2n3bins(2); % mean = upperbound of the lowest bin
                sigma = (BinsOrSet.V2n3bins(3)-BinsOrSet.V2n3bins(2))/sqrt(1-2/pi); % Stdev = range of right neighboring bin divided by a difference factor
                Ran = mu + sigma .* randn(T,1); % Generate full-normally distributed numbers
                V3 = BinsOrSet.V2n3bins(2) - abs(BinsOrSet.V2n3bins(2) - Ran); % Convert full-normal to half-normal
            elseif (m == BinsOrSet.N_V3) % Apply half-Guassian distribution. Only sample the right to the mean. 
                mu = BinsOrSet.V2n3bins(m); % mean = lowerbound of the highest bin
                sigma = (BinsOrSet.V2n3bins(m) - BinsOrSet.V2n3bins(m-1))/sqrt(1-2/pi); % Stdev = range of the left neighboring bin divided by a difference factor
                Ran = mu + sigma .* randn(T,1); % Generate full-normally distributed numbers
                V3 = BinsOrSet.V2n3bins(m) + abs(BinsOrSet.V2n3bins(m) - Ran); % Convert full-normal to half-normal
            else % Sample uniformly in the medium three bins
                V3 = BinsOrSet.V2n3bins(m) + (BinsOrSet.V2n3bins(m+1)...
                    - BinsOrSet.V2n3bins(m)) .* rand(T,1);
            end
            
            for k = 1:BinsOrSet.N_V1 % for each value of V1
                V1_prev = BinsOrSet.V1set(k) * ones(T,1); % Repeat T times
                
                for j = 1:BinsOrSet.N_V1 % for each value of V1
                    V1 = BinsOrSet.V1set(j) * ones(T,1); % Repeat T times
                    
                    for i = 1:params.N_statisticD % for each bin
                        statisticD_prev = BinsOrSet.statisticDbins(i)+...
                            (BinsOrSet.statisticDbins(i+1) - ...
                             BinsOrSet.statisticDbins(i)) * rand(T,1); % Uniformly sample in the bin
                       
                         
                         statisticD = params.mem.gamma_Def * statisticD_prev...
                                     + sign(V1-V1_prev).*sign(V3-V3_prev);
                             
                         [Num,p] = histc(statisticD, BinsOrSet.statisticDbins); % Map back to a bin
%                          size(CPT_statisticD)
%                          [i,j,k,m,n,T]
%                          size(Num(1:end-1,1)/T)
                         CPT_statisticD(:,i,j,k,m,n) = Num(1:end-1,1)/T; % Conditional probabilities
                    end
                end
            end
        end
    end
end
                             
            
            
            
            
            
            
            
            
            