function A = Attacker_policy_d(Attacker, MA, CPTs, params, BinsOrSet)
% Simulate the attacker's policy
% Inputs:    Attacker = attacker
%            OA = attacker's observation
%            MA = attacker's memory
%            global params
%            Bins of q3
% Output:    q3index = Attacker's move in index

if Attacker.exist == 0 % Attacker does not exist
    q3index = median(1:BinsOrSet.N_q3);
% If attacker exists but no region change, then repeat the move    
elseif ( params.event == 1 && MA.EA == MA.EA_prev && rand()<0.9)
    q3index = MA.q3index_prev;
else % Attacker exist
    % Fixed move
    if ( Attacker.mode == 1 )
        q3index = Attacker.thetaA;
    % Randomize uniformly
    elseif ( Attacker.mode == 2 )
        q3index = randi(BinsOrSet.N_q3);
    % Scott's policy
    elseif (Attacker.mode == 3)
        % Do a weighted sample to determine q3index
        q3index = Weighted_sample(CPTs.ScottAttL0(:, MA.V2index, MA.q3index_prev));
    % Stochastic CPT policy
    elseif (Attacker.mode == 4)
        %q3index = Weighted_sample(Attacker.pi(MA.V2index, MA.V3index,:));
        q3index = Weighted_sample(Attacker.pi(MA.V2index, MA.V3index, MA.SAindex, MA.q3index_prev, :));
    % Yildiray's policy
    elseif (Attacker.mode == 5)
        if (MA.V2index==3)
            q3index = MA.m-1;
            if q3index > 11
                q3index = 11;
            elseif q3index < 1;
                q3index = 1;
            end
        elseif (MA.V2index==4 || MA.V2index==5)
            q3index=11;
        else
            q3index=1;
        end
    % David's policy - play maximum push towards edge
    elseif (Attacker.mode == 6)
        if (MA.V2index >= ceil((BinsOrSet.N_V2+1)/2))
            q3index = BinsOrSet.N_q3;
        else
            q3index = 1;
        end
    end
end

A.p3 = params.p3;
A.q3index = q3index;
end