function dXindex = Defender_policy_d(Defender, MD, CPTs, params, BinsOrSet)
% Simulate the defender's policy
% Inputs:    Defender = defender
%            MD = defender's memory
%            CPTs
%            size of the set of dX
%            Global parameters
% Output:    dX = Defender's move

% Choose the policy to play according to Defender.mode
% Parameters to the policy are generally in Defender.thetaD

% If no region change
if (params.event==1 && MD.ED == MD.ED_prev )
    dXindex = MD.Dindex_prev;
else
    % Fixed move
    if ( Defender.mode == 1 )
        dXindex = Defender.thetaD+1;
    % Randomize uniformly in the dX set
    elseif ( Defender.mode == 2 )
        dXindex = randi(BinsOrSet.N_dX); % Index of dX in the set of dX
    % Scott's Defender's L0 policy
    elseif (Defender.mode == 3)        
        % Do a weighted sample to determine dX
        if ( params.routeSD2D0 == 1 ) % Don't route
            dXindex = Weighted_sample(CPTs.ScottDefL0(:,MD.V1index, MD.V2index, MD.V3index));
        elseif ( params.routeSD2D0 == 2 ) % Route
            dXindex = Weighted_sample(CPTs.ScottDefL0(:,MD.V1index, MD.V2index, MD.V3index, MD.SDindex));
        else
            assert(false);
        end
    % Discrete CPT player policy
    elseif ( Defender.mode == 4 )
        dXindex = Weighted_sample(Defender.pi(MD.V1index, MD.V2index, MD.V3index,MD.SDindex,MD.Dindex_prev,:));
    else    
        assert(false);
    end
end

assert(dXindex > 0);