function [Bnew] = Sample_slice_d( B, Defender, Attacker, CPTs, params, BinsOrSet )
% Sample the next time step given the previous one
% Inputs: B = current time step
%         Defender = defender
%         Attacker = attacker
%         CPTs for V1, V2, V3
%         Global parameters
%         Bins or sets of various nodes
% Output: Bnew = next time step
    
    % No previous, use priors
    if ( isempty(B) )
        % Set priors.
        V1prior = params.V0; % V1 is in the base voltage @ t=0
        S_prev.V1index = find(BinsOrSet.V1set == V1prior);%randi(BinsOrSet.N_V1);% % The index of V1@t=0 in the set of V1
        D_prev =  ceil((BinsOrSet.N_dX+1)/2); %randi(3);% dX = 0 @ t=0
        A_prev.p3 = params.p3;
        A_prev.q3index = ceil((BinsOrSet.N_q3+1)/2);% %randi(11) % m = 0 @ t=0
        MD_prev.SDindex = ceil((params.N_statisticD+1)/2);
        MD_prev.V1index = S_prev.V1index; %ceil((BinsOrSet.N_V1+1)/2);
        MD_prev.V2index = ceil((BinsOrSet.N_V2+1)/2);
        MD_prev.V3index = ceil((BinsOrSet.N_V3+1)/2);
        MA_prev.SAindex = ceil((params.N_statisticA+1)/2);
        MA_prev.V3index = ceil((BinsOrSet.N_V3+1)/2);
        MA_prev.V3index_prev = ceil((BinsOrSet.N_V3+1)/2);
        MA_prev.q3index = A_prev.q3index; %ceil((BinsOrSet.N_q3+1)/2);
        MA_prev.q3index_prev = A_prev.q3index; %ceil((BinsOrSet.N_q3+1)/2);
        MA_prev.EA = 2;
    else
        % Unpack previous time step
        [S_prev, MD_prev, MA_prev, D_prev, A_prev, L_prev]...
                            = Slice_unpack_d(B);
    end
    
    % Make sure we're dealing with a discrete player
    assert( params.player_type == 2 );
    
    % Sample next step
    S = Sample_S_d( S_prev, D_prev, A_prev, CPTs, params, BinsOrSet );
    OD = Sample_OD( S ); % defender's observation
    MD = Sample_MD_d(MD_prev, OD, D_prev, CPTs, params ); % defender's memory
    D = Defender_policy_d(Defender, MD, CPTs, params, BinsOrSet ); % Defender's move
    OA = Sample_OA( S ); % attacker's observation
    MA = Sample_MA_d(MA_prev, OA, A_prev, CPTs, BinsOrSet, params); % attacker's memory
    A = Attacker_policy_d(Attacker, MA, CPTs, params, BinsOrSet ); % Attacker's move
    % Pack as new
    Bnew = Slice_pack_d(S, MD, MA, D, A, S.L);
    
end

function [ODnew] = Sample_OD( S )
% Sample defender's observation @ t
% Inputs: V1index, V2index, V3index @ t
% Output: ODnew

    ODnew.V1index = S.V1index;
    ODnew.V2index = S.V2index;
    ODnew.V3index = S.V3index;

end

function [OAnew] = Sample_OA( S )
% Sample attacker's observation @ t
% Inputs: V2index, V3index @ t
% Output: OAnew

    OAnew.V2index = S.V2index;
    OAnew.V3index = S.V3index;
    
end


    


    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
        