%$$ The following code uses Simulated Annealing for finding the optimum
%$$ values for the grid paramaters that will maximize a given social
%$$ welfare

clc
clear all
close all

%$ Select the parameters on which the optimization will run
%$$
startSA = tic;
p2max = (1.40:0.04:2.60);
p3max = (0.60:0.04:2.00);
[p2max_in,p3max_in] = ndgrid(p2max,p3max); %generate the grid
[FirstDim,SecondDim]=size(p2max_in);       %size of optimization space
NumPoint = FirstDim*SecondDim;

%$ Initialization
i=1;
s_test = [randi(FirstDim), randi(SecondDim)]; %initial condition
%s_test = [1,1]; %initial condition
s_repeat = 0;
s_best = [1,1]; %randomly selected
SW_best = -100; %randomly assigned
T = 1; %initial temperature
t(1) = T;
cr = 0.9; %cooling rate 0.95
K=zeros(FirstDim,SecondDim); %This is the #of times the grid points are visited
K(s_test(1),s_test(2)) = 1;


%$$ Simulated annealing algorithm
%$$

%First calculate the SW for the initial point
params = Set_params();
params.p2max = p2max_in(s_test(1),s_test(2));
params.p3max = p3max_in(s_test(1),s_test(2));

datetime = int2str(datevec(now));      %current date and time
datetime(ismember(datetime,' ')) = []; %remove spaces

filename = [datetime '_ECPD-D1A0J-DP0.500-p3max' ...
    num2str(params.p3max,'%.3f') '-p2max' num2str(params.p2max,'%.3f')];

save(strcat('Batch/input_params',sprintf('%0.4d', i)),'params', 'filename');
fprintf('\nProcessing file ');
fprintf(filename);
fprintf('...');

UpdateEnv(params);
ppack.params = params;
ppack = Load_mats(ppack);

[~,SW,SW_SEM] = TrainD1JandSave_SW(ppack,filename);
close all

%below code is used to pause the simulation if needed
statcode=exist('pausesim.rtf');
if statcode==2,
    delete('pausesim.rtf');
    keyboard
end

SW_test = SW;

SW_new = SW_test;
SW_old = SW_new;
s_new=s_test;
s_old=s_new;
sbest = s_new;

%preallocating tracking vectors
t=zeros(1,110);
SW_test_lst = zeros(1,110);
deltaSW_soln = zeros(1,110);
prob = zeros(1,110);
SW_soln = zeros(1,110);

%tracking
s(1,:)=s_new;
stest_lst(1,:)=s_test; %list of s_test
SW_test_lst(1) = SW_test;
deltaSW_soln(1)=0;
prob(1)=1;
SW_soln(1)=SW_new;
t(1)=T;

s_test;
s_new;
sbest;
SW_new;
SW_best;
restart = 0;
num_restart = 0;
while (i<=200);
    clear params
    clear ppack
    i=i+1
    if (s_repeat>10) %restart condition
        %T = t(1); %if temp is desired to be reset after restart then uncomment this line
        s_test = [randi(FirstDim), randi(SecondDim)];
        num_restart = num_restart + 1;
        s_repeat = 0;
        restart = 1;
    else
        s_test(1) = s_old(1)+randi(7)-4; %new first coordinate
        s_test(2) = s_old(2)+randi(7)-4; %new second coordinate
    end
    
    if (mod(i,5)==0) %cooling criteria
        T=T*cr;
    end
    t(i)=T;
    
    if s_test(1)>FirstDim            %saturation function
        s_test(1)=FirstDim;
    elseif s_test(1)<1
        s_test(1)=1;
    end
    
    if s_test(2)>SecondDim;      %saturation function
        s_test(2)=SecondDim;
    elseif s_test(2)<1
        s_test(2)=1;
    end
    
    K(s_test(1),s_test(2)) = K(s_test(1),s_test(2))+1;
    
    params = Set_params();
    params.p2max = p2max_in(s_test(1),s_test(2));
    params.p3max = p3max_in(s_test(1),s_test(2));
    
    % Creating the parameter file
    if (K(s_test(1),s_test(2))==1)
        datetime = int2str(datevec(now));      %current date and time
        datetime(ismember(datetime,' ')) = []; %remove spaces
        filename = [datetime '_ECPD-D1A0J-DP0.500-p3max' ...
            num2str(params.p3max,'%.3f') '-p2max' num2str(params.p2max,'%.3f')];
        
        save(strcat('Batch/input_params',sprintf('%0.4d', i)),'params', 'filename');
        
        fprintf('\nProcessing file ');
        fprintf(filename);
        fprintf('...');
        
        UpdateEnv(params);
        
        %below code is to pause the sim if needed
        statcode=exist('pausesim.rtf','file');
        if statcode==2,
            delete('pausesim.rtf');
            keyboard
        end
        
        ppack.params = params;
        ppack = Load_mats(ppack);
        [~,SW,SW_SEM] = TrainD1JandSave_SW(ppack,filename);
        close all
        SW_test = SW;
    else
        p2max_test = p2max_in(s_test(1),s_test(2));
        p3max_test = p3max_in(s_test(1),s_test(2));
        fname = ['_ECPD-D1A0J-DP0.500-p3max' num2str(params.p3max,'%.3f') ...
            '-p2max' num2str(params.p2max,'%.3f')];
        Corfname = dir(['Saves/*' fname '.mat']);
        load(Corfname.name);
        assert((p2max_test==ppack.params.p2max && p3max_test==ppack.params.p3max) ...
            ,'paramaters do not match');
        SW_test = AvgSocWelf;
    end
    deltaSW = SW_test-SW_old;
    if (deltaSW>0 || restart==1)       %THE ALGORITHM
        s_new = s_test;
        SW_new = SW_test;
        s_repeat=0;
        restart=0;
    elseif exp(deltaSW/T) > rand()
        s_new = s_test;
        SW_new = SW_test;
        s_repeat=0;
    else
        s_repeat = s_repeat+1;
        SW_new = SW_old;
        s_new = s_old;
    end
    
    
    if SW_new>SW_best
        SW_best=SW_new;
        sbest=s_new;
    end
    SW_old = SW_new;
    s_old = s_new;
    
    %for tracking
    s(i,:)=s_new;
    stest_lst(i,:)=s_test;
    SW_test_lst(i) = SW_test;
    deltaSW_soln(i)=deltaSW;
    prob(i)=exp(deltaSW/T);
    SW_soln(i)=SW_new;
    
    
    s_test;
    s_new;
    sbest;
    SW_new;
    SW_best;
    prob(i)
    if mod(i,20)==0
        SAname = ['SA_' int2str(i)];
        save(SAname);
    end
    close all
end

endSAtime = toc(startSA);

datetime = int2str(datevec(now));      %current date and time
datetime(ismember(datetime,' ')) = []; %remove spaces
fnamee=[datetime '_simann'];
save(fnamee);
fprintf('\nLoadAndRun Done!\n\n');


