%$$ The following code uses Simulated Annealing for finding the optimum
%$$ values for the grid paramaters that will maximize a given social
%$$ welfare

clc
clear all
close all

%$ L0 Attacker
%$$
L0Attacker.mode = 3;
L0Attacker.thetaA = 0.07;
L0Attacker.exist=1;


%$ Select the parameters on which the optimization will run
%$$
p2max = [1.40, 1.50, 1.60];
p3max = [0.6, 0.8, 1.0, 1.2];
probexist=[0.1 0.5 1.0];
[p2max_in,p3max_in,probexist_in] = ndgrid(p2max,p3max,probexist); %generate the grid
[FirstDim,SecondDim,ThirdDim]=size(p2max_in);       %size of optimization space
NumPoint = FirstDim*SecondDim*ThirdDim;

%$ Initialization
i=1;
s_test = [randi(FirstDim), randi(SecondDim), randi(ThirdDim)] %initial state
s_repeat=1;
s_best = [1,1,1]; %randomly selected
SW_best = -100;
T = 1; %initial temperature
t(1) = T;
cr = 0.8; %cooling rate
K=zeros(FirstDim,SecondDim,ThirdDim); %This is the #of times the grid points are visited
K(s_test(1),s_test(2),s_test(3)) = 1;

%$$ Simulated annealing algorithm
%$$

%First calculate the SW for the initial point
p2max_test = p2max_in(s_test(1),s_test(2),s_test(3));
p3max_test = p3max_in(s_test(1),s_test(2),s_test(3));
probexist_test = probexist_in(s_test(1),s_test(2),s_test(3));
fname = ['AE01DP' num2str(probexist_test,'%.3f') '-p3max' num2str(p3max_test,'%.3f') ...
    '-p2max' num2str(p2max_test,'%.3f')];
Corfname = dir(['Saves_060112/*' fname '.mat']);
load(Corfname.name);
ppack.params.event=2;

fprintf('Average reward in continuos...')
[defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM, ...
    defAvgNumMove, defAvgNumMoveSEM, AvgSocWelf, AvgSocWelfSEM ]...
    = Average_rewards( L1Defender, L0Attacker, 50, false,ppack);
SW_test = AvgSocWelf;
% assert((p2max_new==ppack.params.p2max && p3max_new==ppack.params.p3max) ...
%     ,'paramaters do not match');
% params = Set_params();
% params.p2max = p2max_in(s(1),s(2));
% params.p3max = p3max_in(s(1),s(2));
%
% filename = sprintf('ECPD-D1A0J-p2max%0.3f-p3max%0.3f', params.p2max, ...
%     params.p3max);
% save(strcat('Batch/input_params',sprintf('%0.4d', i)),'params', 'filename');
% fprintf('\nProcessing file ');
% fprintf(num2str(i));
% fprintf('...');
%
% UpdateEnv(params);
% ppack.params = params;
% ppack = Load_mats(ppack);
% [~,SW,SW_SEM] = TrainD1JandSave_SW(ppack,filename);

SW_new = SW_test;
SW_old = SW_new;
s_new=s_test;
s_old=s_new;
sbest = s_new;

%tracking

s(1,:)=s_new;
stest_lst(1,:)=s_test; %list of s_test
SW_test_lst(1) = SW_test;
deltaSW_soln(1)=0;
prob(1)=1;
SW_soln(1)=SW_new;
t(1)=T;


s_test
s_new
sbest
SW_new
SW_best

while (i<=NumPoint && s_repeat<10);
    i=i+1
    if (mod(i,3)==0) %if a new data point was reached in the previous step
        T=T*cr;
    end
    t(i)=T;
    s_test(1) = s_old(1)+randi(3)-2; %new first coordinate
    s_test(2) = s_old(2)+randi(3)-2; %new second coordinate
    s_test(3) = s_old(3)+randi(3)-2; %new third coordinate
    if s_test(1)>FirstDim         %saturation function
        s_test(1)=FirstDim;
    elseif s_test(1)<1
        s_test(1)=1;
    end
    
    if s_test(2)>SecondDim;      %saturation function
        s_test(2)=SecondDim;
    elseif s_test(2)<1
        s_test(2)=1;
    end
    
    if s_test(3)>ThirdDim;      %saturation function
        s_test(3)=ThirdDim;
    elseif s_test(3)<1
        s_test(3)=1;
    end
    
    K(s_test(1),s_test(2),s_test(3)) = K(s_test(1),s_test(2),s_test(3))+1;
  
    p2max_test = p2max_in(s_test(1),s_test(2),s_test(3));
    p3max_test = p3max_in(s_test(1),s_test(2),s_test(3));
    probexist_test = probexist_in(s_test(1),s_test(2),s_test(3));
    fname = ['AE01DP' num2str(probexist_test,'%.3f') '-p3max' num2str(p3max_test,'%.3f') ...
        '-p2max' num2str(p2max_test,'%.3f')];
    Corfname = dir(['Saves_060112/*' fname '.mat']);
    load(Corfname.name);
    %     assert((p2max_new==ppack.params.p2max && p3max_new==ppack.params.p3max) ...
    %         ,'paramaters do not match');
    fprintf('Average reward in continuos...')
    ppack.params.event=2;
    [defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM, ...
        defAvgNumMove, defAvgNumMoveSEM, AvgSocWelf, AvgSocWelfSEM ]...
        = Average_rewards( L1Defender, L0Attacker, 50, false,ppack);
    SW_test = AvgSocWelf;
    
    deltaSW = SW_test-SW_old;
    if deltaSW>0                %THE ALGORITHM
        s_new = s_test;
        SW_new = SW_test;
    elseif exp(deltaSW/T) > rand()
        s_new = s_test;
        SW_new = SW_test;
    else
        s_repeat = s_repeat+1;
        SW_new = SW_old;
        s_new = s_old;
    end
    
    if SW_new>SW_best
        SW_best=SW_new;
        sbest=s_new;
    end
    SW_old = SW_new;
    s_old = s_new;
    
    %for tracking
    s(i,:)=s_new;
    s_test_lst(i,:)=s_test;
    SW_test_lst(i) = SW_test;
    deltaSW_soln(i)=deltaSW;
    prob(i)=exp(deltaSW/T);
    SW_soln(i)=SW_new;
    
    
    s_test
    s_new
    sbest
    SW_new
    SW_best
    prob(i)
    
    close all
end

for i=2:size(prob,2)
    if prob(i)>=1
        prob(i)=prob(i-1);
    end
end

fprintf('\nLoadAndRun Done!\n\n');



