%$$ The following code uses Exhaustive search for finding the optimum
%$$ values for the grid paramaters that will maximize a given social
%$$ welfare

clc
clear all
close all

%$ Select the parameters on which the optimization will run
%$$
startSA = tic;
p2max = [1.4 1.6]; %(1.40:0.04:2.60);
p3max = (0.60:0.20:1.40);
[p2max_in,p3max_in] = ndgrid(p2max,p3max); %generate the grid
[FirstDim,SecondDim]=size(p2max_in);       %size of optimization space
NumPoint = FirstDim*SecondDim;

%preallocation
s = zeros(NumPoint,2);
SW_track = zeros(FirstDim,SecondDim);
SW_track_single = zeros(1,NumPoint);
SW_SEM_track = zeros(FirstDim,SecondDim);
SW_SEM_track_single = zeros(1,NumPoint);

k=1;
for i=1:FirstDim
    for j=1:SecondDim
        i
        j
        
        clear params
        clear ppack
        
        params = Set_params();
        params.p2max = p2max_in(i,j);
        params.p3max = p3max_in(i,j);
        
        % Creating the parameter file
        datetime = int2str(datevec(now));      %current date and time
        datetime(ismember(datetime,' ')) = []; %remove spaces
        filename = [datetime '_ECPD-D1A0J-DP0.500-p3max' ...
            num2str(params.p3max,'%.3f') '-p2max' num2str(params.p2max,'%.3f')];
        
        save(strcat('Batch/input_params',sprintf('%0.4d', (i-1)*SecondDim+j)),'params', 'filename');
        
        fprintf('\nProcessing file ');
        fprintf(filename);
        fprintf('...');
        
        %UpdateEnv(params);
        
        ppack.params = params;
        ppack = Load_mats(ppack); %just assigned something to prevent error warnings
        ppack.BinsOrSet = Save_BinsOrSet(params);
        [~,SW,SW_SEM] = TrainD1JandSave_SW(ppack,filename);
        close all
        
        %for tracking
        s(k,:) = [i j];
        SW_track(i,j) = SW;
        SW_track_single(k) = SW;
        SW_SEM_track(i,j) = SW_SEM;
        SW_SEM_track_single(k) = SW_SEM;
        k=k+1;
        
        if mod(k,50)==0
            SAname = ['SA_' int2str(i)];
            save(SAname);
        end
        close all
        
        %below code is to pause the sim if needed
        statcode=exist('pausesim.rtf','file');
        if statcode==2,
            delete('pausesim.rtf');
            keyboard
        end
    end
end

endSAtime = toc(startSA);

datetime = int2str(datevec(now));      %current date and time
datetime(ismember(datetime,' ')) = []; %remove spaces
fnamee=[datetime '_ExhaustiveSearch'];
save(fnamee);

[SW_max,ind] = max(SW_track(:))
[ii,jj] = ind2sub(size(p2max_in),ind);
s_max = [ii jj]
fprintf('\nLoadAndRun Done!\n\n');



