%$$ The following code uses Exhaustive search for finding the optimum
%$$ values for the grid paramaters that will maximize a given social
%$$ welfare

    
    clc
    clear all
    close all
    
    HiFi = 1; % HiFi: original discrete training. Set this to 0 otherwise
    
    
    %$ Select the parameters on which the optimization will run
    %$$
    startSA = tic;
    p2max = 1.5; %[1.72 2.04 2.36]; %(1.40:0.04:2.60);
    p3max = 1.2; %[0.96 1.32 1.68]; %(0.60:0.04:2.00);
    [p2max_in,p3max_in] = ndgrid(p2max,p3max); %generate the grid
    [FirstDim,SecondDim]=size(p2max_in);       %size of optimization space
    NumPoint = FirstDim*SecondDim;
    
    %preallocation
    s = zeros(NumPoint,2);
    SW_track = zeros(FirstDim,SecondDim);
    SW_track_single = zeros(1,NumPoint);
    SW_SEM_track = zeros(FirstDim,SecondDim);
    SW_SEM_track_single = zeros(1,NumPoint);
    
    k=1;
    for i=1:FirstDim
        for j=1:SecondDim
            i
            j
            
            clear params
            clear ppack
            
            params = Set_params();
            params.p2max = p2max_in(i,j);
            params.p3max = p3max_in(i,j);
            
            % Creating the parameter file
            datetime = int2str(datevec(now));      %current date and time
            datetime(ismember(datetime,' ')) = []; %remove spaces
            filename = [datetime '_ECPD-D1A0J-DP0.500-p3max' ...
                num2str(params.p3max,'%.3f') '-p2max' num2str(params.p2max,'%.3f')];
            
            save(strcat('Batch/input_params',sprintf('%0.4d', (i-1)*SecondDim+j)),'params', 'filename');
            
            fprintf('\nProcessing file ');
            fprintf(filename);
            fprintf('...');
            
            ppack.params = params;
            
            if HiFi
                %UpdateEnv(params);
                ppack = Load_mats(ppack);
                ppack.useContL0 = 0;
            else
                ppack.useContL0 = 1;
            end
            
            ppack.BinsOrSet = Save_BinsOrSet(params);
            
            [~,SW,SW_SEM, AAR, AAR_SEM, DAR, DAR_SEM] = TrainD1JandSave_SW2(ppack,filename);
            close all
            
            %for tracking
            s(k,:) = [i j];
            SW_track(i,j) = SW;
            SW_track_single(k) = SW;
            SW_SEM_track(i,j) = SW_SEM;
            SW_SEM_track_single(k) = SW_SEM;
            AAR_track(i,j) = AAR;
            AAR_track_single(k) = AAR;
            AAR_SEM_track(i,j) = AAR_SEM;
            AAR_SEM_track_single(k) = AAR_SEM;
            DAR_track(i,j) = DAR;
            DAR_track_single(k) = DAR;
            DAR_SEM_track(i,j) = DAR_SEM;
            DAR_SEM_track_single(k) = DAR_SEM;
            k=k+1;
            
            close all
            
            %below code is to pause the sim if needed
            statcode=exist('pausesim.rtf','file');
            if statcode==2,
                delete('pausesim.rtf');
                keyboard
            end
        end
    end
    
    endSAtime = toc(startSA);
    
    datetime = int2str(datevec(now));      %current date and time
    datetime(ismember(datetime,' ')) = []; %remove spaces
    fnamee=[datetime '_ExhaustiveSearch'];
    save(fnamee);
    
    [SW_max,ind] = max(SW_track(:))
    [ii,jj] = ind2sub(size(p2max_in),ind);
    s_max = [ii jj]


fprintf('\nLoadAndRun Done!\n\n');



