%$$ The following code uses Exhaustive search for finding the optimum
%$$ values for the grid paramaters that will maximize a given social
%$$ welfare

clc
clear all
close all

% Select the range of parameters for SW to be optimized over
p2max_min = 1.4;
p2max_max = 2.6;
p3max_min = 0.6;
p3max_max = 2.0;

% Ranges of data points for optimization
rangeY = p3max_min:0.04:p3max_max;%min(p3max_in):.04:max(p3max_max);
rangeX = p2max_min:0.04:p2max_max;%min(p2max_min):.04:max(p2max_max);

HiFi = 1; % HiFi: original discrete training. Set this to 0 otherwise


%$ Select the initial data points to create the first surrogate
%$$
startSG = tic;

startSA = tic;
p2max = [1.4 1.72 2.04 2.36 2.60]; %(1.40:0.04:2.60);
p3max = [0.6 0.96 1.32 1.68 2.00]; %(0.60:0.04:2.00);
[p2max_in,p3max_in] = ndgrid(p2max,p3max); %generate the grid
[FirstDim,SecondDim]=size(p2max_in);       %size of optimization space
NumPoint = FirstDim*SecondDim;

%preallocation
s = zeros(NumPoint,2);
SW_track = zeros(FirstDim,SecondDim);
SW_track_single = zeros(1,NumPoint);
SW_SEM_track = zeros(FirstDim,SecondDim);
SW_SEM_track_single = zeros(1,NumPoint);
AAR_track = zeros(FirstDim,SecondDim);
AAR_track_single = zeros(1,NumPoint);
AAR_SEM_track = zeros(FirstDim,SecondDim);
AAR_SEM_track_single = zeros(1,NumPoint);
DAR_track = zeros(FirstDim,SecondDim);
DAR_track_single = zeros(1,NumPoint);
DAR_SEM_track = zeros(FirstDim,SecondDim);
DAR_SEM_track_single = zeros(1,NumPoint);

soln = zeros(size(rangeX,2)*size(rangeY,2)/2,2);
test_soln = zeros(size(rangeX,2)*size(rangeY,2)/2,2);
SW_soln = zeros(1,size(rangeX,2)*size(rangeY,2)/2);
SW_test = zeros(1,size(rangeX,2)*size(rangeY,2)/2);
mean_p2p3 = zeros(size(rangeX,2)*size(rangeY,2)/2,2);

k=1;
for i=1:FirstDim
    for j=1:SecondDim
        i
        j

        clear params
        clear ppack
        close all
        
        params = Set_params();
        params.p2max = p2max_in(i,j);
        params.p3max = p3max_in(i,j);

        % Creating the parameter file
        datetime = int2str(datevec(now));      %current date and time
        datetime(ismember(datetime,' ')) = []; %remove spaces
        filename = [datetime '_ECPD-D1A0J-DP0.500-p3max' ...
            num2str(params.p3max,'%.3f') '-p2max' num2str(params.p2max,'%.3f')];

        save(strcat('Batch/input_params',sprintf('%0.4d', (i-1)*SecondDim+j)),'params', 'filename');

        fprintf('\nProcessing file ');
        fprintf(filename);
        fprintf('...');

        ppack.params = params;

        if HiFi
            UpdateEnv(params);
            ppack = Load_mats(ppack);
            ppack.useContL0 = 0;
        else
            ppack.useContL0 = 1;
        end

        ppack.BinsOrSet = Save_BinsOrSet(params);

        [~,SW,SW_SEM, AAR, AAR_SEM, DAR, DAR_SEM] = TrainD1JandSave_SW2(ppack,filename);
        close all

        %for tracking
        s(k,:) = [i j];
        SW_track(i,j) = SW;
        SW_track_single(k) = SW;
        SW_SEM_track(i,j) = SW_SEM;
        SW_SEM_track_single(k) = SW_SEM;
        AAR_track(i,j) = AAR;
        AAR_track_single(k) = AAR;
        AAR_SEM_track(i,j) = AAR_SEM;
        AAR_SEM_track_single(k) = AAR_SEM;
        DAR_track(i,j) = DAR;
        DAR_track_single(k) = DAR;
        DAR_SEM_track(i,j) = DAR_SEM;
        DAR_SEM_track_single(k) = DAR_SEM;
        k=k+1;
        
        close all

        %below code is to pause the sim if needed
        statcode=exist('pausesim.rtf','file');
        if statcode==2,
            delete('pausesim.rtf');
            keyboard
        end
    end
end

endSAtime = toc(startSA);

datetime = int2str(datevec(now));      %current date and time
datetime(ismember(datetime,' ')) = []; %remove spaces
fnamee=[datetime '_Surrogate'];
save(fnamee);

%
% Load the first surrogate
 SurName = dir('*Surrogate.mat');
 load(SurName.name);


% Create the first surrogate surface

p2max_list = p2max_in(:);
p3max_list = p3max_in(:);
SW_track_list = SW_track(:);

figure
p_h = plot3(p2max_list,p3max_list,SW_track_list,'ro','LineWidth',6);
hold on


[X,Y] = meshgrid(rangeX,rangeY);
Z = griddata(p2max_list,p3max_list,SW_track_list,X,Y,'v4');
surf(X,Y,Z)
grid
hold off

SW_best = -realmax;
Repeat_test = 0;
m = 1;
soln_old = [0 0];

%$$ Adding new data points and creating new surrogates until convergence
%$$

while Repeat_test < 15 %this should be around 30?
    %find the max point in the current surrogate
    [max_SW,ind] = max(Z(:))
    max_Xind = mod(ind,size(Z,1));
    if max_Xind == 0
        max_Xind = size(Z,1);
    end
    max_Yind = floor(ind/size(Z,1))+1;
    max_p2max = X(max_Xind,max_Yind)
    max_p3max = Y(max_Xind,max_Yind)
    soln(m,:) = [max_p2max max_p3max]; %current solution
    SW_soln(m) = max_SW;
    
    if soln(m,1)==soln_old(1) && soln(m,2)==soln_old(2)
        Repeat_test = Repeat_test + 1
    else
        Repeat_test = 0;
    end
    
    soln_old = soln(m,:);
    
    %$determine the next point for the mean of the gaussian
    
%     %determining the mean using relative SW values
%     Z_nneg = Z(:)-min(Z(:));
%     [~,indZ]=histc(rand(1),[0;cumsum(Z_nneg)/sum(Z_nneg)]);
%     mean_Xind = mod(indZ,size(Z,1));
%     if mean_Xind == 0
%         mean_Xind = size(Z,1);
%     end
 
    
    %determining the mean using relative SW values where only the top
    %1/3 of the SW values are considered
    Z_nneg = Z(:)-min(Z(:));
    Z_nntoprange = max(Z_nneg(:))-(max(Z_nneg(:))-min(Z_nneg(:)))/3;
    ind_LowZnn = find(Z_nneg(:)<Z_nntoprange);
    Znn_mod = Z_nneg;
    Znn_mod(ind_LowZnn) = 0;
    
    [~,indZ]=histc(rand(1),[0;cumsum(Znn_mod)/sum(Znn_mod)]);
    mean_Xind = mod(indZ,size(Z,1));
    if mean_Xind == 0
        mean_Xind = size(Z,1);
    end
    mean_Yind = floor(indZ/size(Z,1))+1;
    mean_p2max = X(mean_Xind,mean_Yind);
    mean_p3max = Y(mean_Xind,mean_Yind);
    mean_p2p3(m,:) = [mean_p2max mean_p3max];
    
    
    %determine the next point to be tested
    cond_both = 1;
    cond_p2 = 1;
    cond_p3 = 1;
    while cond_both==1 || cond_p2==1 || cond_p3==1
        if cond_both==1 || cond_p2==1
            p2max_test = mean_p2max + (p2max_max-p2max_min)/3*randn;
        end
        if cond_both==1 || cond_p3==1
            p3max_test = mean_p3max + (p3max_max-p3max_min)/3*randn;
        end
        cond_p2 = 0;
        cond_p3 = 0;
        if (p2max_test > p2max_max) || (p2max_test < p2max_min) %border checking
            cond_p2 = 1;
        end
        if (p3max_test > p3max_max) || (p3max_test < p3max_min)
            cond_p3 = 1;
        end
        test_soln(m,:) = [p2max_test p3max_test];
        if m>1
            cond_both = 0;
            for i=1:(m-1)
                if test_soln(m,1)==test_soln(i,1) && test_soln(m,2)==test_soln(i,2)
                    cond_both = 1;
                end
            end
        else
            cond_both = 0;
        end
    end
    
    
    
    %$$ Calculate the SW for the test point
    %$$
    
    clear params
    clear ppack
    close all
    
    params = Set_params();
    params.p2max = p2max_test;
    params.p3max = p3max_test;
    
    % Creating the parameter file
    datetime = int2str(datevec(now));      %current date and time
    datetime(ismember(datetime,' ')) = []; %remove spaces
    filename = [datetime '_ECPD-D1A0J-DP0.500-p3max' ...
        num2str(params.p3max,'%.3f') '-p2max' num2str(params.p2max,'%.3f')];
    
    save(strcat('Batch/input_params',sprintf('%0.4d', (i-1)*SecondDim+j)),'params', 'filename');
    
    fprintf('\nProcessing file ');
    fprintf(filename);
    fprintf('...');
    
    ppack.params = params;
    
    if HiFi
        UpdateEnv(params);
        ppack = Load_mats(ppack);
        ppack.useContL0 = 0;
    else
        ppack.useContL0 = 1;
    end
    
    ppack.BinsOrSet = Save_BinsOrSet(params);
    
    [~,SW,SW_SEM, AAR, AAR_SEM, DAR, DAR_SEM] = TrainD1JandSave_SW2(ppack,filename);
    close all
    
    SW_test(m) = SW;
    
    %create the new surrogate
    SW_track_list = [SW_track_list' SW_test(m)]';
    p2max_list = [p2max_list' p2max_test]';
    p3max_list = [p3max_list' p3max_test]';
      
    Z = griddata(p2max_list,p3max_list,SW_track_list,X,Y,'v4');
  
    m=m+1;
    
    statcode=exist('pausesim.rtf','file');
    if statcode==2,
        delete('pausesim.rtf');
        keyboard
    end
    if mod(m,5)==0
        save('TempSur');
    end
end

%find the max point in the final surrogate
[max_SW,ind] = max(Z(:))
max_Xind = mod(ind,size(Z,1));
if max_Xind == 0
    max_Xind = size(Z,1);
end
max_Yind = floor(ind/size(Z,1))+1;
max_p2max = X(max_Xind,max_Yind)
max_p3max = Y(max_Xind,max_Yind)
soln(m,:) = [max_p2max max_p3max]; %current solution
SW_soln(m) = max_SW;

figure
p_h = plot3(p2max_list,p3max_list,SW_track_list,'mo','LineWidth',4);
hold on
surf(X,Y,Z)
grid
hold off

endSGtime = toc(startSG);

datetime = int2str(datevec(now));      %current date and time
datetime(ismember(datetime,' ')) = []; %remove spaces
fnamee=[datetime '_Surrogate_final'];
save(fnamee);







fprintf('\nLoadAndRun Done!\n\n');



