%Train an L1 defender using Jaakkola

function filename = TrainA1JandSave( ppack, filename )
clc


if ( ~exist('ppack') )
    ppack.params = Set_params();
    ppack = Load_mats(ppack);
end
if ( ~exist('filename') )
    filename = [];
end

close all;

L0Defender.mode = ppack.params.L0Def_mode;

% Modifying Scott's L0 Attacker to be used as policy initialization in
% Jaakkola training.
ScottL0Attinit= Scott_L0Att_forinit(ppack.params, ppack.BinsOrSet, ppack.CPTs);

fprintf('\nTrain L1 Attacker...')
method = 2; % 1: InitL0-NoObsNoTrain, 2: InitRand-NoObsFillL0
tic
L1Attacker = Train_attacker_Jaakkola( L0Defender, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER, 1, ScottL0Attinit, method);
toc

fprintf('\nSimulate L0 Defender vs L1 Attacker in continuous (exist)...')
L1Attacker.exist = 1;
[D0A1states, D0A1defRewards, D0A1attRewards] = Simulate ( L0Defender, L1Attacker, 1, ppack );

fprintf('Average reward in continuos...')
[defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM]... 
                = Average_rewards( L0Defender, L1Attacker, 50, false,ppack);

filename=[];
fprintf('\nSaving...');
filename = SaveDescription( SaveAllOpenPlots(filename), ppack );
save( fullfile('Saves',[filename '.mat']), 'L1Attacker', 'ppack', ...
    'defAvgReward','defAvgRewardSEM','attAvgReward', 'attAvgRewardSEM');

fprintf('\nTrainA1JandSave Done!\n\n');