%Train an L1 defender using Jaakkola
function [filename, SW, SW_SEM] = TrainD1JandSave_SW( ppack, filename )

if ( ~exist('ppack') )
    ppack.params = Set_params();
    ppack = Load_mats(ppack);
end
if ( ~exist('filename') )
    filename = [];
end

close all;

clear L0Attacker;
L0Attacker.mode = ppack.params.L0Att_mode;
L0Attacker.thetaA = 0.07;

% Modifying Scott's L0 Defender to be used as policy initialization in
% Jaakoola training.
%ScottLODefinit= Scott_L0Def_forinit(ppack.params, ppack.BinsOrSet, ppack.CPTs);

fprintf('\nTrain L1 Defender...')
%method = 2; % 1: InitL0-NoObsNoTrain, 2: InitRand-NoObsFillL0
tic
%L1Defender = Train_defender_Jaakkola( L0Attacker, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER, 1, ScottLODefinit, method);
L1Defender = Train_defender_Jaakkola_c_NoPrvInp( L0Attacker, ppack, 1);
%L1Defender = Train_defender_Jaakkola_c( L0Attacker, ppack, 1);
toc

% fprintf('\nSimulate L1 Defender vs L0 Attacker in continuous (does not exist)...')
% %load('2012514171853 ECPD-D1A0J-AE01DP0-p3max0.6-mth1.mat');
% L0Attacker.exist = 0;
% [D1A0states, D1A0defRewards, D1A0attRewards] = Simulate ( L1Defender, L0Attacker, 1, ppack );

fprintf('\nSimulate L1 Defender vs L0 Attacker in continuous (exist)...')
L0Attacker.exist = 1;
[D1A0states, D1A0defRewards, D1A0attRewards] = Simulate ( L1Defender, L0Attacker, 1, ppack );

fprintf('Average reward in continuos...')
[defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM, ...
    defAvgNumMove, defAvgNumMoveSEM, AvgSocWelf, AvgSocWelfSEM] ... 
                = Average_rewards( L1Defender, L0Attacker, 100, false,ppack);
SW = AvgSocWelf;
SW_SEM = AvgSocWelfSEM;

fprintf('\nSaving...');
filename = SaveDescription( SaveAllOpenPlots(filename), ppack );
save( fullfile('Saves',[filename '.mat']), 'L1Defender','ppack', ...
    'defAvgReward','defAvgRewardSEM','attAvgReward', 'attAvgRewardSEM', ...
    'defAvgNumMove', 'defAvgNumMoveSEM', 'AvgSocWelf', 'AvgSocWelfSEM');

fprintf('\nTrainD1JandSave Done!\n\n');