function [Attacker] = Train_attacker_Jaakkola(Defender, params, BinsOrSet, CPTs, ER, makeplot, ScottL0Attinit, method)
%Learn a RL Defender assuming given an Attacker
%using Jaakkola algorithm.
%This version uses epoch-wise updating of the policy.
%Input: Attacker = attacker
%Output: Defender = learned RL defender
%global params;

fprintf('Jaakkola_attacker...\n')

params.jaak = Set_params_Jaak();
Attacker.trainParams = params.jaak;

%create a new defender
clear Attacker;
Attacker.mode = 4;     %Put attacker into policy mode
Attacker.exist=1;

trackIndex = 0; %for tracking

%Initializing the Jaakkola matrices and the average reward
r_av = 0;
Attacker.K = ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA,BinsOrSet.N_q3,BinsOrSet.N_q3);
Attacker.beta = ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA,BinsOrSet.N_q3,BinsOrSet.N_q3);
Attacker.Q = 20*ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA,BinsOrSet.N_q3,BinsOrSet.N_q3);
Attacker.C = ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA,BinsOrSet.N_q3,BinsOrSet.N_q3);

% Attacker.K_msg = ones(BinsOrSet.N_V2,BinsOrSet.N_V3);
% Attacker.beta_msg = ones(BinsOrSet.N_V2,BinsOrSet.N_V3);
% Attacker.V = ones(BinsOrSet.N_V2,BinsOrSet.N_V3);
% Attacker.C_msg = ones(BinsOrSet.N_V2,BinsOrSet.N_V3);

Attacker.gamma_Q=params.jaak.gamma; %ones(BinsOrSet.N_V2,BinsOrSet.N_V3,BinsOrSet.N_q3)*params.jaak.gamma;
Attacker.gamma_V=params.jaak.gamma; %ones(BinsOrSet.N_V2,BinsOrSet.N_V3)*params.jaak.gamma;


%Initializing the stochastic Defender Policy Defender.pi(a|m) and greedy
%policy pi1(a|m) (see the Jaakkola paper for explanations)

if (method == 1) % InitL0-NoObsNoTrain
    Attacker.pi=ScottL0Attinit;
end

if (method == 2) % InitRand-NoObsFillL0
    Attacker.pi=ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA,BinsOrSet.N_q3,BinsOrSet.N_q3)/(BinsOrSet.N_q3);
end
%pi1=ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA,BinsOrSet.N_q3,BinsOrSet.N_q3)/(BinsOrSet.N_q3);


%Initializing tracking matrices

Attacker.deltaQ=ones(1,1000);
delta_old=1;

Attacker.avreward=zeros(1,5000);
Attacker.avreward_in=zeros(1,5000);

%B = Sample_slice_d([], Defender, Attacker, CPTs, params, BinsOrSet);
ep=1;

Attacker.pi_old=Attacker.pi*2;

a_t_old = 5;

K_old = Attacker.K; %This is used to see which states are actually observed
                    %by comparing old and new K values.

Attacker.MeanInLoRewAv_old (1) = 0;

while ep<params.jaak.nTrainEpsAtt

    kl(ep)=max(max(max(max(sum(Attacker.pi_old.*log(Attacker.pi_old./Attacker.pi),5)))));
    
    ep
    
    Attacker.pi_old = Attacker.pi;
    
    B = Sample_slice_d([], Defender, Attacker, CPTs, params, BinsOrSet);
    
    InnerLoopRewAv = 0;
    
    for t = 1:params.jaak.NumInnerStepsAtt %params.jaak.nSimSteps
        [S_t,MD_t,MA_t,dX_t,A_t,L_t] = Slice_unpack_d(B);
        
        V1_t = S_t.V1index;
        V2_t = S_t.V2index;
        V3_t = S_t.V3index;
        a_t = A_t.q3index; %a_t is the index of q3
        SA_t = MA_t.SA;
        
        %calculate the reward
        
        r = ER.Att(V2_t);
        
        
        %Updating Jaakkola message-action matrices M(m,a) for m_t=m, a_t=a
        
        
        Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t)=Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t) + 1;
        
        Attacker.beta=Attacker.gamma_Q*Attacker.beta;
        Attacker.Q=Attacker.Q+Attacker.beta*(r-r_av);
        
        Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t) = Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t) - Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t)*(r-r_av);
        Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t)=(1-(1/Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t)))*Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t)+1/Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t);
        Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t)=(1-(1/Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t)))*Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t)+Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t)*(r-r_av);
        
        
        %Update the average reward estimate with the current reward
        r_av_new = r_av + (r-r_av)/(params.jaak.NumInnerStepsAtt*(ep-1)+t);
        InnerLoopRewAv = InnerLoopRewAv + (r-InnerLoopRewAv)/t;
        
        %Transform Jaakkola message-action matrices M(m,a) for m_t=m,
        %a_t=a using the new average reward estimate
        
        
        Attacker.C = Attacker.C + Attacker.beta;
        Attacker.Q = Attacker.Q - Attacker.C*(r_av_new-r_av);
        
        Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t) = Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t) + Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t)*(r_av_new-r_av);
        Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t) = Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t) - Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t);
        
        Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t) = (1-(1/Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t)))*Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t)+Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t);
        Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t) = Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t)-Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t)*(r_av_new-r_av);
        
        r_av = r_av_new;
        a_t_old = a_t;
        
        %Sample the next step
        B = Sample_slice_d(B, Defender, Attacker, CPTs, params, BinsOrSet);
        
        
        %For tracking
        trackIndex = trackIndex + 1;
        
    end
    
    %Update the Attacker Policy Attacker.pi(a|m) at the end of the episode
    %by determining the greedy policy pi1 and then updating the Attacker
    %policy towards the greedy policy
    
    
    Attacker.pi=(1-params.jaak.epsln)*Attacker.pi;
    
    Q_temp = Attacker.Q;
    m = max(Q_temp,[],5);
    t = bsxfun(@eq,m,Q_temp); % Identify all multiple maxima along each dim 5
    f = find(t); % Get pointers to all of them
    
    indexx = f;
    
    if (0) %make this 1 if you want to randomly pick a maximum and update it
        %0 is you want to simply update all maximums
        
        t = t*rand; %For some reason the following line does not work without this line
        t(f) = rand(length(f),1); % Multiply each 'TRUE' by a random number
        [~,ind] = max(t,[],5); % Find the maximums of these random numbers
        [I,J,K,P] = ind2sub(size(ind),1:size(ind(:)));
        
        indexx = sub2ind(size(Attacker.pi),I,J,K,P,ind(:)');
    end
    
    if (method == 1)
        idx_K = find(sum(Attacker.K,5)==sum(K_old,5));
        Leidx_K = length(idx_K);
        RepLeidx = ones(1,Leidx_K);
        
        K_order = RepLeidx;
        for i=2:BinsOrSet.N_q3
            K_order = cat(2,K_order,RepLeidx*i);
        end
        K_old = Attacker.K;
        [I_K,J_K,K_K,P_K] = ind2sub(size(Attacker.K),idx_K);
        
        ind_K = sub2ind(size(Attacker.K),repmat(I_K,BinsOrSet.N_q3,1),repmat(J_K,BinsOrSet.N_q3,1), ...
            repmat(K_K,BinsOrSet.N_q3,1),repmat(P_K,BinsOrSet.N_q3,1), K_order');
        
        [~,idx] = setdiff(indexx,ind_K);
        indexx = indexx(sort(idx));
    end
    
    Attacker.pi(indexx) = ...
        Attacker.pi(indexx) + params.jaak.epsln*1;
    
    normz = repmat(sum(Attacker.pi,5), [1, 1, 1, 1, BinsOrSet.N_q3]);
    Attacker.pi = Attacker.pi ./ normz;
    
    
    
    %For tracking
    Attacker.InnerLoopRewAv(ep) = InnerLoopRewAv;
    
    Attacker.avreward(ep)=r_av;
    r_av
    
%     sumkl = sum(Attacker.pi_old.*log((Attacker.pi_old+10^(-5))./(Attacker.pi+(10^(-5)))),5);
%     sument = -sum(Attacker.pi.*log2(Attacker.pi+10^(-5)),5);
%     kl= max(sumkl(:))*10^7
%     ent= max(sument(:))
    
    ep=ep+1;
end

if makeplot
    figure
    plot(Attacker.InnerLoopRewAv);
    title('Running window average reward');
    
    figure
    plot(Attacker.avreward);
    title('Average reward');
    
    figure
    plot(kl);
    title('Kullback?Leibler divergence');
end

end

%$$ Some code that may be useful later
%$$

%Stopping criteria for training
%     while (max(max(max(max(sum(Attacker.pi_old.*log(Attacker.pi_old./Attacker.pi),5)))))*10^7>5 ...
%     && max(max(max(max(-sum(Attacker.pi.*log2(Attacker.pi),5)))))>0.8);   %min(Attacker.pi(:))>0.03
%     while SameRew<100000 && ep<3000

%Extract variables at t+1
%     [S_tp1,MD_t,MA_t,dX_tp1,A_tp1,L_tp1] = Slice_unpack_d(B);
%     V1_tp1 = S_tp1.V1index;
%     V2_tp1 = S_tp1.V2index;
%     V3_tp1 = S_tp1.V3index;
%     a_tp1 = A_tp1.q3index; %a_tp1 is the index of q3_tp1

%Delta Q and Delta pi computation
%     Q_old = Attacker.Q;   %This will be used for Delta_Q computation
%     Attacker.pi_old = Attacker.pi; %This will be used for Delta_pi computation

