function [Defender] = Train_defender_Jaakkola(Attacker, ppack, makeplot, ScottL0Definit, method)
%Learn a RL Defender assuming given an Attacker
%using Jaakkola algorithm.
%This version uses epoch-wise updating of the policy.
%Input: Attacker = attacker
%Output: Defender = learned RL defender


%%$$$  Updates
%%$$$
% 04/09/2012 -- Added AD(t-1) and SD(t-1) in the RL algorithm



fprintf('Jaakkola_defender...\n')

makeHist = 1; %To make a hist of the visited states, set to 1, otherwise 0;

params = ppack.params;
BinsOrSet = ppack.BinsOrSet;
CPTs = ppack.CPTs;
ER = ppack.ER;

params.jaak = Set_params_Jaak();
Defender.trainParams = params.jaak;

%create a new defender
clear Defender;
Defender.mode = 4;     %Put defender into policy mode

trackIndex = 0; %for tracking

%Initializing the Jaakkola matrices and the average reward
r_av = 0;
Defender.K = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX);
Defender.beta = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX);
Defender.Q = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX);
Defender.C = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX);

%These may be used in the future so keep it here in the code
%  Defender.K_msg = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX);
%  Defender.beta_msg = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX);
%  Defender.V = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX);
%  Defender.C_msg = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX);

Defender.gamma_Q=params.jaak.gamma; %ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,BinsOrSet.N_dX)*params.jaak.gamma;
Defender.gamma_V=params.jaak.gamma; %ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3)*params.jaak.gamma;


%Initializing the stochastic Defender Policy Defender.pi(a|m) and greedy
%policy pi1(a|m) (see the Jaakkola paper for explanations)


if (method == 1) % InitL0-NoObsNoTrain
    Defender.pi=ScottL0Definit;
elseif (method == 2) % InitRand-NoObsFillL0
    Defender.pi=ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX)/(BinsOrSet.N_dX);
elseif (method == 3) %if one wants to use a certain Defender as the initial Defender
    Defender.pi = ScottL0Definit.pi; %ScottL0Defender name is used to prevent adding more variables
    Defender.Q = ScottL0Definit.Q;
else
    Defender.pi=ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX)/(BinsOrSet.N_dX);
end



%Initializing tracking matrices

Defender.deltaQ=ones(1,1000);
delta_old=1;

Defender.avreward=zeros(1,5000);
Defender.avreward_in=zeros(1,5000);

if(makeHist)
    Defender.trainVisitedStates = zeros(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX);
end

Attacker.exist=1;
%B = Sample_slice_d([], Defender, Attacker, CPTs, params, BinsOrSet);
ep=1;

Defender.pi_old=Defender.pi*2;
a_t_old = 2;

K_old = Defender.K; %This is used to see which states are actually observed
%by comparing old and new K values.

Defender.MeanInLoRewAv_old (1) = 0; % Running average for the defender reward
check_ind = 0;

while ep<params.jaak.nTrainEpsDef
%while ep<2
    ep
    Defender.kl_matrix = sum(Defender.pi_old.*log(Defender.pi_old./(Defender.pi+2.2e-16)+2.2e-16),6);
    Defender.kl(ep) = mean(Defender.kl_matrix(:));
    Defender.pi_old = Defender.pi;
    
    % coin flipping on attacker's existence
    Attacker.exist = genAttExists(params.ProbAttExist);
    
    % if he exists, coin flip again to determine personality
    if ( Attacker.exist == 1 && isfield(Attacker,'switchmode') && ...
            isfield(Attacker,'switchthetaA') && isfield(Attacker,'switchprobs'))
        id = Weighted_sample( Attacker.switchprobs );
        Attacker.mode = Attacker.switchmode(id);
        Attacker.thetaA = Attacker.switchthetaA(id);
    end
    
    B = Sample_slice_d([], Defender, Attacker, CPTs, params, BinsOrSet);
    
    InnerLoopRewAv = 0;
    
    for t = 1:params.jaak.NumInnerStepsDef %params.jaak.nSimSteps
        
        [S_t,MD_t,MA_t,dX_t,A_t,L_t] = Slice_unpack_d(B);
        
        V1_t = S_t.V1index;
        V2_t = S_t.V2index;
        V3_t = S_t.V3index;
        a_t  = dX_t;      % defenders action at time t
        SD_t = MD_t.SDindex;   % statisticsD at time t
        
        %Set this to 1 if you want to track a histogram of space
        if(makeHist)
            Defender.trainVisitedStates(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)...
                = Defender.trainVisitedStates(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)+1;
        end
        
        %calculate the reward
        r = ER.Def(V2_t,V3_t);
        
        
        %Updating Jaakkola message-action matrices M(m,a) for m_t=m, a_t=a
        
        Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)=Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) + 1;
        Defender.beta=Defender.gamma_Q*Defender.beta;
        Defender.Q=Defender.Q+Defender.beta*(r-r_av);
        
        Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) = Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) - Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)*(r-r_av);
        Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)=(1-(1/Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)))*Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)+1/Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t);
        Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)=(1-(1/Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)))*Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)+Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)*(r-r_av);
        
        
        %Update the average reward estimate with the current reward
        %r_av_new = r_av + (r-r_av)/t;
        r_av_new = r_av + (r-r_av)/(params.jaak.NumInnerStepsDef*(ep-1)+t);
        InnerLoopRewAv = InnerLoopRewAv + (r-InnerLoopRewAv)/t;
        
        %Transform Jaakkola message-action matrices M(m,a) for m_t=m,
        %a_t=a using the new average reward estimate
        
        Defender.C = Defender.C + Defender.beta;
        Defender.Q = Defender.Q - Defender.C*(r_av_new-r_av);
        
        Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) = Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) + Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)*(r_av_new-r_av);
        Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) = Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) - Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t);
        
        Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) = (1-(1/Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)))*Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)+Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t);
        Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) = Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)-Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)*(r_av_new-r_av);
        
        r_av = r_av_new;
        a_t_old = a_t;
        
        %Sample the next step
        B = Sample_slice_d(B, Defender, Attacker, CPTs, params, BinsOrSet);
        
        %For tracking
        trackIndex = trackIndex + 1;
        
        Defender.avreward_in(t)=r_av;
        
        if(0) %Set to 1 to save full training history
            Defender.trainHistory{ep}{t} = B;
        end
    end
    
    %Update the Defender Policy Defender.pi(a|m) at the end of the episode
    %by determining the greedy polict pi1 and then updating the Defender
    %policy towards the greedy policy
    
    Defender.pi=(1-params.jaak.epsln)*Defender.pi;
    
    Q_temp = Defender.Q;
    m = max(Q_temp,[],6);
    t = bsxfun(@eq,m,Q_temp); % Identify all multiple maxima along each dim 6
    f = find(t); % Get pointers to all of them
    
    indexx = f;
    
    if (0) %make this 1 if you want to randomly pick a maximum and update it
        %0 is you want to simply update all maximums
        
        t = t*rand; %For some reason the following line does not work without this line
        t(f) = rand(length(f),1); % Multiply each 'TRUE' by a random number
        [~,ind] = max(t,[],6); % Find the maximums of these random numbers
        [I,J,K,P,R] = ind2sub(size(ind),1:size(ind(:)));
        
        indexx = sub2ind(size(Defender.pi),I,J,K,P,R,ind(:)');
    end
    
    
    
    
    if (method == 1)
        idx_K = find(sum(Defender.K,6)==sum(K_old,6));
        Leidx_K = length(idx_K);
        RepLeidx = ones(1,Leidx_K);
        K_order = [RepLeidx RepLeidx*2 RepLeidx*3];
        K_old = Defender.K;
        [I_K,J_K,K_K,P_K,R_K] = ind2sub(size(Defender.K),idx_K);
        
        ind_K = sub2ind(size(Defender.K),repmat(I_K,3,1),repmat(J_K,3,1), ...
            repmat(K_K,3,1),repmat(P_K,3,1),repmat(R_K,3,1), K_order');
        
        [~,idx] = setdiff(indexx,ind_K);
        indexx = indexx(sort(idx));
    end
    
    Defender.pi(indexx) = ...
        Defender.pi(indexx) + params.jaak.epsln*1;
    
    normz = repmat(sum(Defender.pi,6), [1, 1, 1, 1, 1, BinsOrSet.N_dX]);
    Defender.pi = Defender.pi ./ normz;
    
    %For tracking
    Defender.avreward(ep)=r_av;
    Defender.InnerLoopRewAv(ep) = InnerLoopRewAv;
    
    
    sumkl = sum(Defender.pi_old.*log(Defender.pi_old./Defender.pi),6);
    sument = -sum(Defender.pi.*log2(Defender.pi),6);
    %     kl = max(sumkl(:))*10^7
    %     ent = max(sument(:))
    
    
    
    if mod(ep,40) == 1
        check_ind = check_ind + 1;
        fprintf('Average reward in continuous...')
        [defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM, ...
            defAvgNumMove, defAvgNumMoveSEM, AvgSocWelf, AvgSocWelfSEM] ...
            = Average_rewards( Defender, Attacker, 100, false, ppack);
        
        Defender.defAvgReward_RealCheck(check_ind) = defAvgReward;
        Defender.defAvgRewardSEM_RealCheck(check_ind) = defAvgRewardSEM;
    end
    
    
    ep=ep+1;
    
end

if (method == 2)
    idx_K = find(sum(Defender.K,6)<7);
    Leidx_K = length(idx_K);
    RepLeidx = ones(1,Leidx_K);
    K_order = [RepLeidx RepLeidx*2 RepLeidx*3];
    [I_K,J_K,K_K,P_K,R_K] = ind2sub(size(Defender.K),idx_K);
    ind_K = sub2ind(size(Defender.K),repmat(I_K,3,1),repmat(J_K,3,1), ...
        repmat(K_K,3,1),repmat(P_K,3,1),repmat(R_K,3,1), K_order');
    Defender.pi(ind_K)=ScottL0Definit(ind_K);
end


if makeplot
    figure
    plot(Defender.InnerLoopRewAv);
    title('Running window average reward');
    
    figure
    plot(Defender.avreward);
    title('Average reward');
    
    figure
    plot(Defender.defAvgReward_RealCheck);
    title('Real Average reward');
    
    figure
    errorbar(Defender.defAvgReward_RealCheck,Defender.defAvgRewardSEM_RealCheck);
    title('Real Average reward with SEM');
end


end

%$$ Code that may be useful later but not used now
%$$

%Extract variables at t+1
%        [S_tp1,MD_tp1,MA_tp1,dX_tp1,A_tp1,L_tp1] = Slice_unpack_d(B);

%         V1_tp1 = S_tp1.V1index;
%         V2_tp1 = S_tp1.V2index;
%         V3_tp1 = S_tp1.V3index;
%         a_tp1 = dX_tp1;
%         SD_tp1 = MD_tp1.SD;   % statisticsD at time t


%Different stopping criteria for the traning
%         while (max(max(max(max(max(sum(Defender.pi_old.*log(Defender.pi_old./Defender.pi),6))))))*10^7>5 ...
%         && max(max(max(max(max(-sum(Defender.pi.*log2(Defender.pi),6))))))>0.4);

%Used for calculating delta_Q and delta_Pi
%         Q_old_in = Defender.Q;   %This will be used for Delta_Q computation
%         pi_old_in = Defender.pi; %This will be used for Delta_pi computation
%         delta=sum(sum(sum(sum(sum(sum(abs(Defender.Q)-abs(Q_old_in)))))));
%         Defender.deltaQ(t) = delta_old-delta;
%         delta_old=delta;
%         Q_old = Defender.Q;   %This will be used for Delta_Q computation
%         Defender.pi_old = Defender.pi; %This will be used for Delta_pi computation

