function [Attacker] = Train_attacker_Jaakkola(Defender, ppack, makeplot )
%Learn a RL Attacker assuming given an Defender
%using Jaakkola algorithm.
%This version uses epoch-wise updating of the policy.
%Input: Defender = defender
%Output: Attacker = learned RL attacker

params = ppack.params;
BinsOrSet = ppack.BinsOrSet;

fprintf('Jaakkola_attacker...\n')

params.jaak = Set_params_Jaak_c();
Attacker.trainParams = params.jaak;

%create a new defender
clear Attacker;
Attacker.mode = 4;     %Put attacker into policy mode
Attacker.exist=1;

trackIndex = 0; %for tracking

%Initializing the Jaakkola matrices and the average reward
r_av = 0;
Attacker.K = ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA,BinsOrSet.N_q3,BinsOrSet.N_q3);
Attacker.beta = ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA,BinsOrSet.N_q3,BinsOrSet.N_q3);
Attacker.Q = 20*ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA,BinsOrSet.N_q3,BinsOrSet.N_q3);
Attacker.C = ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA,BinsOrSet.N_q3,BinsOrSet.N_q3);

Attacker.gamma_Q=params.jaak.gamma;
Attacker.gamma_V=params.jaak.gamma;

%Initializing the stochastic Defender Policy Defender.pi(a|m) and greedy
%policy pi1(a|m) (see the Jaakkola paper for explanations)

Attacker.pi=ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA,BinsOrSet.N_q3,BinsOrSet.N_q3)/(BinsOrSet.N_q3);

%Initializing tracking matrices
Attacker.deltaQ=ones(1,1000);
delta_old=1;

Attacker.avreward=zeros(1,5000);
Attacker.avreward_in=zeros(1,5000);

%B = Sample_slice_d([], Defender, Attacker, CPTs, params, BinsOrSet);
ep=1;

Attacker.pi_old=Attacker.pi;
a_t_old = 5;

K_old = Attacker.K; %This is used to see which states are actually observed
                    %by comparing old and new K values.

Attacker.MeanInLoRewAv_old (1) = 0;

while ep < params.jaak.nTrainEpsAtt
    ep
    Attacker.kl_matrix = sum(Attacker.pi_old.*log(Attacker.pi_old./(Attacker.pi+2.2e-16)+2.2e-16),5);
    Attacker.kl(ep) = mean(Attacker.kl_matrix(:));
    Attacker.pi_old = Attacker.pi;
    
    B = Sample_slice([], Defender, Attacker, ppack);
    
    InnerLoopRewAv = 0;
    for t = 1:params.jaak.NumInnerStepsAtt %params.jaak.nSimSteps
        [S_t,OD_t,OA_t,MD_t,MA_t,Dindex_t,Aindex_t] = Slice_unpack(B);
        
        V2_t = MA_t.V2index;
        V3_t = MA_t.V3index;
        a_t = Aindex_t.q3index; %a_t is the index of q3
        SA_t = MA_t.SA;
        
        %calculate the reward
        r = Attacker_reward(B, params);
        
        %Updating Jaakkola message-action matrices M(m,a) for m_t=m, a_t=a
        
        Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t)=Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t) + 1;
        Attacker.beta=Attacker.gamma_Q*Attacker.beta;
        Attacker.Q=Attacker.Q+Attacker.beta*(r-r_av);
        
        Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t) = Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t) - Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t)*(r-r_av);
        Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t)=(1-(1/Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t)))*Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t)+1/Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t);
        Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t)=(1-(1/Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t)))*Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t)+Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t)*(r-r_av);
        
        
        %Update the average reward estimate with the current reward
        r_av_new = r_av + (r-r_av)/(params.jaak.NumInnerStepsAtt*(ep-1)+t);
        InnerLoopRewAv = InnerLoopRewAv + (r-InnerLoopRewAv)/t;
        
        %Transform Jaakkola message-action matrices M(m,a) for m_t=m,
        %a_t=a using the new average reward estimate
        
        Attacker.C = Attacker.C + Attacker.beta;
        Attacker.Q = Attacker.Q - Attacker.C*(r_av_new-r_av);
        
        Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t) = Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t) + Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t)*(r_av_new-r_av);
        Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t) = Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t) - Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t);
        
        Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t) = (1-(1/Attacker.K(V2_t,V3_t,SA_t,a_t_old,a_t)))*Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t)+Attacker.beta(V2_t,V3_t,SA_t,a_t_old,a_t);
        Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t) = Attacker.Q(V2_t,V3_t,SA_t,a_t_old,a_t)-Attacker.C(V2_t,V3_t,SA_t,a_t_old,a_t)*(r_av_new-r_av);
        
        r_av = r_av_new;
        a_t_old = a_t;
        
        %Sample the next step
        B = Sample_slice(B, Defender, Attacker, ppack);
        
        %For tracking
        trackIndex = trackIndex + 1;
     
        Attacker.avreward_in(t)=r_av;
        
        if(0) %Set to 1 to save full training history
            Attacker.trainHistory{ep}{t} = B;
        end
    end
    
    %Update the Attacker Policy Attacker.pi(a|m) at the end of the episode
    %by determining the greedy policy pi1 and then updating the Attacker
    %policy towards the greedy policy
       
    Attacker.pi=(1-params.jaak.epsln)*Attacker.pi;
    
    Q_temp = Attacker.Q;
    m = max(Q_temp,[],5);
    t = bsxfun(@eq,m,Q_temp); % Identify all multiple maxima along each dim 5
    f = find(t); % Get pointers to all of them
    
    indexx = f;
    
    if (0) %make this 1 if you want to randomly pick a maximum and update it
        %0 is you want to simply update all maximums
        
        t = t*rand; %For some reason the following line does not work without this line
        t(f) = rand(length(f),1); % Multiply each 'TRUE' by a random number
        [~,ind] = max(t,[],5); % Find the maximums of these random numbers
        [I,J,K,P] = ind2sub(size(ind),1:size(ind(:)));
        
        indexx = sub2ind(size(Attacker.pi),I,J,K,P,ind(:)');
    end
    
    Attacker.pi(indexx) = ...
        Attacker.pi(indexx) + params.jaak.epsln*1;
    
    normz = repmat(sum(Attacker.pi,5), [1, 1, 1, 1, BinsOrSet.N_q3]);
    Attacker.pi = Attacker.pi ./ normz;
    
    %For tracking
    Attacker.avreward(ep) = r_av;
    Attacker.InnerLoopRewAv(ep) = InnerLoopRewAv;
    
%     sumkl = sum(Attacker.pi_old.*log(Attacker.pi_old./Attacker.pi),5);
%     sument = -sum(Attacker.pi.*log2(Attacker.pi),5);
    
    ep=ep+1;
end

if makeplot
    figure
    plot(Attacker.InnerLoopRewAv);
    title('Running window average reward');
    
    figure
    plot(Attacker.avreward);
    title('Average reward');
end

end
