function [Attacker] = Train_Attacker_OptQlearning(Defender,params,BinsOrSet,CPTs,ER)
% This function is used to train the attacker with optimistic Q-learning

    fprintf('Optimistic_Qlearning Attacker...\n')
    
    % Create a new attacker
    clear Attacker;
    Attacker.exist = 1;
    Attacker.mode = params.policy_mode; % Put attacker into policy mode
    
    % Initialize tracker; num of times an (o,m,a) has been visited
    Att_oma_t = zeros(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA, BinsOrSet.N_q3);
    
    R = sort(ER.Att); % sort the matrix
    Rmax = R(BinsOrSet.N_V2, 1); % maximum reward for the attacker
    Q0 = Rmax/(1-params.gammaA); % initial Q for all (s,a)
    
    Attacker.Qtable = Q0 * ones(BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticA, BinsOrSet.N_q3);
    
    % initialization time
    %T = params.c*((Vmax^2/params.epsilonD^2 * (log(1/params.deltaD)+...
                  %log(Vmax/params.epsilonD)))^(1/params.Def_learn_omega));
    
    for ep = 1:params.nTrainEpsAtt_Q
        
        % randomize an (o,m,a)-tuple as the starter
        S.V2index = randi(BinsOrSet.N_V2);
        S.V3index = randi(BinsOrSet.N_V3);
        MA.SA = randi(params.N_statisticA);
        
        % find an optimal action
        [Qmax,q3star] = max(Attacker.Qtable(S.V2index,S.V3index,MA.SA,:));
        Astar.p3 = params.p3;
        Astar.q3index = q3star; 
        
        % start a Bnet
        S.V1index = randi(BinsOrSet.N_V1);
        D = randi(BinsOrSet.N_dX);
        MD.V1index = S.V1index;
        MD.V3index = S.V3index;
        MD.SD = randi(params.N_statisticD);
        MA.V3index = S.V3index;
        MA.V3index_prev = randi(BinsOrSet.N_V3);
        MA.q3index = q3star;
        MA.q3index_prev = randi(BinsOrSet.N_q3);
        L = 0;
        B = Slice_pack_d(S,MD,MA,D,Astar,L);
        
        for t = 1:params.nSimSteps
            [S,~,MA,~,A,~] = Slice_unpack_d(B);
            
            Bnew = Sample_slice_d(B,Defender,Attacker,CPTs,params,BinsOrSet);
            [Snew,~,MAnew,~,~,~] = Slice_unpack_d(Bnew);
            
            % find an optimal action
            [Qmax_new,q3star_new] = max(Attacker.Qtable(Snew.V2index,Snew.V3index,MAnew.SA,:));
            
            % update the tracker
            Att_oma_t(Snew.V2index,Snew.V3index,MAnew.SA,q3star_new)=...
                Att_oma_t(Snew.V2index,Snew.V3index,MAnew.SA,q3star_new) + 1;
            
            % reward
            r = ER.Att(Snew.V2index, 1);
            
            % current learning rate
            alpha = 1/(1 + Att_oma_t(Snew.V2index,Snew.V3index,...
                      MAnew.SA,q3star_new)^params.Att_learn_omega);
                  
            Attacker.Qtable(S.V2index,S.V3index,MA.SA,A.q3index)...
            = Qmax + alpha * (r + params.gammaA*Qmax_new - Qmax);
        
            Qmax = Qmax_new;
            B = Bnew;
        end
    end
end
            
            
            
            
        
        