function [Defender] = Train_Defender_OptQlearning(Attacker,params,BinsOrSet,CPTs,ER)
% This function is used to train the defender with optimistic Q-learning


    fprintf('Optimistic_Qlearning Defender...\n')
    
    % Create a new defender
    clear Defender;
    Defender.mode = params.policy_mode; % Put defender into policy mode
    
    % Initialize tracker; num of times an (o,m,a) has been visited
    Def_oma_t = zeros(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,...
                      params.N_statisticD, BinsOrSet.N_dX);
    
    R = sort(sort(ER.Def)'); % sort the matrix
    Rmax = R(BinsOrSet.N_V2,BinsOrSet.N_V3); % maximum reward for the defender 
    Q0 = Rmax/(1-params.gammaD); % initial Q for all (s,a)
    
    Defender.Qtable = Q0 * ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,...
                      params.N_statisticD, BinsOrSet.N_dX);
    
    % initialization time
    %T = params.c*((Vmax^2/params.epsilonD^2 * (log(1/params.deltaD)+...
                  %log(Vmax/params.epsilonD)))^(1/params.Def_learn_omega));
    
    for ep = 1:params.nTrainEpsDef_Q
        % coin flipping on attacker's existence
        Attacker.exist = genAttExists(params.ProbAttExist);
        
        % randomize an (o,m,a)-tuple as the starter
        S.V1index = randi(BinsOrSet.N_V1);
        S.V2index = randi(BinsOrSet.N_V2);
        S.V3index = randi(BinsOrSet.N_V3);
        MD.SD = randi(params.N_statisticD);
        
        % find an optimal action
        [Qmax,Dstar] = max(Defender.Qtable(S.V1index,S.V2index,S.V3index,MD.SD,:));
        
        % start a Bnet
        A.p3 = params.p3;
        A.q3index = randi(BinsOrSet.N_q3);
        MD.V1index = S.V1index;
        MD.V3index = S.V3index;
        MA.SA = randi(params.N_statisticA);
        MA.V3index = S.V3index;
        MA.V3index_prev = S.V3index;
        MA.q3index = randi(BinsOrSet.N_q3);
        MA.q3index_prev = randi(BinsOrSet.N_q3);
        L = 0;
        B = Slice_pack_d(S,MD,MA,Dstar,A,L);
                
        for t = 1:params.nSimSteps
            [S,MD,~,D,~,~] = Slice_unpack_d(B);
            
            Bnew = Sample_slice_d(B,Defender,Attacker,CPTs,params,BinsOrSet);
            [Snew,MDnew,~,~,~,~] = Slice_unpack_d(Bnew);     
            
            % find an optimal action
            [Qmax_new,Dstar_new] = max(Defender.Qtable(Snew.V1index,...
                Snew.V2index,Snew.V3index,MDnew.SD,:));
            
            % update the tracker
            Def_oma_t(Snew.V1index,Snew.V2index,Snew.V3index,MDnew.SD,Dstar_new)=...
                Def_oma_t(Snew.V1index,Snew.V2index,Snew.V3index,MDnew.SD,Dstar_new) + 1;
            
            % reward
            r = ER.Def(Snew.V2index, Snew.V3index);
            
            % current learning rate
            alpha = 1/(1 + Def_oma_t(Snew.V1index,Snew.V2index,Snew.V3index,...
                      MDnew.SD,Dstar_new)^params.Def_learn_omega);
                  
            Defender.Qtable(S.V1index,S.V2index,S.V3index,MD.SD,D)...
            = Qmax + alpha * (r + params.gammaD*Qmax_new - Qmax);
        
            Qmax = Qmax_new;
            B = Bnew;
        end
    end
end
            
            
            
            
            
            
            
            
            
            
            