%Use this file as example code snippets for calling Matlab code for 
%three-node circuit example

clc
clear all;
close all;

%Call this first to be able to call functions from sub-directories
addpath(genpath(pwd));

%Must first call these scripts to set parameters and load matrices
ppack.params = Set_params();
ppack = Load_mats(ppack);

L0Attacker.mode = 3;
L0Attacker.thetaA = 0.07;
L0Attacker.exist = 1;

L0Defender.mode = 3;
L0Defender.thetaD = 0;


%%

% Jaakkola A0D1
fprintf('Train L1Defender with Jaakkola')
makeplot = 1;
tic
L1Defender = Train_defender_Jaakkola( L0Attacker, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER, makeplot );
toc
fprintf('Simulate in discrete...')
[D1A0states_d, D1A0defRewards_d, D1A0attRewards_d] = Simulate_d(L1Defender, L0Attacker, 1, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER);
fprintf('Simulate in continuous...')
[D1A0states, D1A0defRewards, D1A0attRewards] = Simulate ( L1Defender, L0Attacker, 1, ppack );

%%

% Jaakkola A1D0
fprintf('Train L1Attacker with Jaakkola')
makeplot = 1;
L1Attacker = Train_attacker_Jaakkola( L0Defender, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER, makeplot );
fprintf('Simulate in discrete...')
[D0A1states_d, D0A1defRewards_d, D0A1attRewards_d] = Simulate_d(L0Defender, L1Attacker, 1, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER);
fprintf('Simulate in continuous...')
[D0A1states, D0A1defRewards, D0A1attRewards] = Simulate ( L0Defender, L1Attacker, 1, ppack );


%%

% Jaakkola A1D2
fprintf('Train L2Defender...')
makeplot = 1;
load('L1Attacker25.mat','L1Attacker');  %load which L1 Attacker you want to train against
tic
L2Defender = Train_defender_Jaakkola( L1Attacker, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER );
toc
%%

% Jaakkola A2D1
fprintf('Train L2Attacker...')
makeplot = 1;
load('L1defender30','L1Defender'); %L1 Defender against which training is done
tic
L2Attacker = Train_attacker_Jaakkola( L1Defender, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER, makeplot );
toc
%%

% Simulate A0D0 in continuos
fprintf('Simulate L0 Defender vs L0 Attacker in continuous...')
[D0A0states, D0A0defRewards, D0A0attRewards] = Simulate ( L0Defender, L0Attacker, 1, ppack );
%%

 Simulate A0D1 in continuos
fprintf('Simulate L1 Defender vs L0 Attacker in continuous...')
%load('L1defender30.mat','L1Defender') %L1 Defender to be simulated
[D1A0states, D1A0defRewards, D1A0attRewards] = Simulate ( L1Defender, L0Attacker, 1, ppack );
%%

% Simulate A1D0 in continuous
fprintf('Simulate L0 Defender vs L1 Attacker in continuous...')
load('L1Attacker25.mat','L1Attacker'); % L1 Attacker to be simulated
[D0A1states, D0A1defRewards, D0A1attRewards] = Simulate ( L0Defender, L1Attacker, 1, ppack );
%%

% Simulate A1D1 in continuous
fprintf('Simulate L1 Defender vs L1 Attacker in continuous...')
load('L1defender30.mat','L1Defender');  %L1 Defender to be simulated
load('L1Attacker10.mat','L1Attacker');  %L1 Attacker to be simulated
[D1A1states, D1A1defRewards, D1A1attRewards] = Simulate ( L1Defender, L1Attacker, 1, ppack );
%%

% Simulate A1D2 in continuous
fprintf('Simulate L2 Defender vs L1 Attacker in continuous...')
load('L2defender29.mat','L2Defender')
load('L1defender30.mat','L1Defender');
[D1A1states, D1A1defRewards, D1A1attRewards] = Simulate ( L2Defender, L1Attacker, 1, ppack );
%%

% Simulate A2D1 in continuous
fprintf('Simulate L1 Defender vs L2 Attacker in continuous...')
load('L1defender30.mat','L1Defender');
load('L2Attacker37.mat','L2Attacker');
[D1A1states, D1A1defRewards, D1A1attRewards] = Simulate ( L1Defender, L2Attacker, 1, ppack );
%%

% Optimistic Q-learning A0D1
fprintf('Train L1Defender with Optimistic Q-learning')
tic 
L1Defender = Train_Defender_OptQlearning(L0Attacker,ppack.params,ppack.BinsOrSet,ppack.CPTs,ppack.ER);
toc
fprintf('Simulate in discrete...')
[D1A0states_d, D1A0defRewards_d, D1A0attRewards_d] = Simulate_d(L1Defender, L0Attacker, 1, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER);
fprintf('Simulate in continuous...')
[D1A0states, D1A0defRewards, D1A0attRewards] = Simulate ( L1Defender, L0Attacker, 1, ppack );

%%

%To simulate N episodes and compute average performances of players, use
fprintf('Average reward in continuos...')
load('L2attacker37','L2Attacker'); %The attacker you want to simulate
load('L1defender30','L1Defender'); %The defender you want to simulate
ppack.params.nSimSteps = 200;
[defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM]... 
                = Average_rewards( L1Defender, L2Attacker, 50, false,ppack);
%%

% Save all open plots into a single pdf, save params and BinsOrSet to text file
% and save to 'Saves' directory
SaveDescription( SaveAllOpenPlots(), ppack )
%%

% Optimistic Q-learning A1D0
fprintf('Train L1Attacker with Optimistic Q-learning')
tic 
L1Attacker = Train_Attacker_OptQlearning(L0Defender,ppack.params,ppack.BinsOrSet,ppack.CPTs,ppack.ER);
toc
fprintf('Simulate in discrete...')
[D0A1states_d, D0A1defRewards_d, D0A1attRewards_d] = Simulate_d(L0Defender, L1Attacker, 1, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER);
fprintf('Simulate in continuous...')
[D0A1states, D0A1defRewards, D0A1attRewards] = Simulate ( L0Defender, L1Attacker, 1, ppack );

%%

% Optimistic Q-learning A1D2
fprintf('Train L1Attacker with Optimistic Q-learning')
tic 
L1Attacker = Train_Attacker_OptQlearning(L0Defender,ppack.params,ppack.BinsOrSet,ppack.CPTs,ppack.ER);
toc
fprintf('Train L2Defender with Optimistic Q-learning')
tic 
L2Defender = Train_Defender_OptQlearning(L1Attacker,ppack.params,ppack.BinsOrSet,ppack.CPTs,ppack.ER);
toc
fprintf('Simulate in discrete...')
[D2A1states_d, D2A1defRewards_d, D2A1attRewards_d] = Simulate_d(L2Defender, L1Attacker, 1, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER);
fprintf('Simulate in continuous...')
[D2A1states, D2A1defRewards, D2A1attRewards] = Simulate ( L2Defender, L1Attacker, 1, ppack );

%%

% Optimistic Q-learning A2D1
fprintf('Train L1Defender with Optimistic Q-learning')
tic 
L1Defender = Train_Defender_OptQlearning(L0Attacker,ppack.params,ppack.BinsOrSet,ppack.CPTs,ppack.ER);
toc
fprintf('Train L2Attacker with Optimistic Q-learning')
tic 
L2Attacker = Train_Attacker_OptQlearning(L1Defender,ppack.params,ppack.BinsOrSet,ppack.CPTs,ppack.ER);
toc
fprintf('Simulate in discrete...')
[D1A2states_d, D1A2defRewards_d, D1A2attRewards_d] = Simulate_d(L1Defender, L2Attacker, 1, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER);
fprintf('Simulate in continuous...')
[D1A2states, D1A2defRewards, D1A2attRewards] = Simulate ( L1Defender, L2Attacker, 1, ppack );
