%Use this file as example code snippets for calling Matlab code for 
%three-node circuit example

clc
clear all;
close all;

%Call this first to be able to call functions from sub-directories
addpath(genpath(pwd));

%Must first call these scripts to set parameters and load matrices
ppack.params = Set_params();
ppack = Load_mats(ppack);

L0Attacker.mode = 3;
L0Attacker.thetaA = 0.07;

L0Defender.mode = 3;
L0Defender.thetaD = 0;

%%
%
fprintf('Simulate L0 Defender vs L0 Attacker in continuous...')
L0Attacker.exist = 1;
[D0A0states, D0A0defRewards, D0A0attRewards] = Simulate ( L0Defender, L0Attacker, 1, ppack );

%%
%
fprintf('Train L1Defender...')
tic
L1Defender = Train_defender_Jaakkola( L0Attacker, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER );
toc

%%
%
fprintf('Train L2Defender...')
load('L1Attacker25.mat','L1Attacker');  %load which L1 Attacker you want to train against
tic
L2Defender = Train_defender_Jaakkola( L1Attacker, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER );
toc

%%
%
fprintf('Simulate L1 Defender vs L0 Attacker in discrete...')
load('L1defender30.mat','L1Defender') %L1 Defender to be simulated
[D1A0states_d, D1A0defRewards_d, D1A0attRewards_d] = Simulate_d(L1Defender, L0Attacker, 1, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER)

%%
%
fprintf('Simulate L1 Defender vs L0 Attacker in continuous...')

load('L1defender30.mat','L1Defender') %L1 Defender to be simulated
[D1A0states, D1A0defRewards, D1A0attRewards] = Simulate ( L1Defender, L0Attacker, 1, ppack );

%%
%
fprintf('Train L1Attacker...')
tic
%L1Attacker = Train_attacker_Jaakkola_memorylessNoPrvInp( L0Defender, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER );
L1Attacker = Train_attacker_Jaakkola( L0Defender, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER );
toc

%%
%
% Training L2 Attacker 
fprintf('Train L2Attacker...')
load('L1defender30','L1Defender'); %L1 Defender against which training is done
tic
L2Attacker = Train_attacker_Jaakkola( L1Defender, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER );
toc
%%
%
close all

fprintf('Simulate L0 Defender vs L1 Attacker in discrete...')

load('L1Attacker25.mat','L1Attacker');  % L1 Attacker to be simulated
[D0A1states_d, D0A1defRewards_d, D0A1attRewards_d] = Simulate_d(L0Defender, L1Attacker, 1, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER)

%%
%
close all

fprintf('Simulate L0 Defender vs L1 Attacker in continuous...')

load('L1Attacker25.mat','L1Attacker'); % L1 Attacker to be simulated
[D0A1states, D0A1defRewards, D0A1attRewards] = Simulate ( L0Defender, L1Attacker, 1, ppack );

%%
%
%Simulating L1 Defender vs L1 Attacker - Continuos
close all

fprintf('Simulate L1 Defender vs L1 Attacker in continuous...')
 
load('L1defender30.mat','L1Defender');  %L1 Defender to be simulated
load('L1Attacker10.mat','L1Attacker');  %L1 Attacker to be simulated
[D1A1states, D1A1defRewards, D1A1attRewards] = Simulate ( L1Defender, L1Attacker, 1, ppack );

%%
%
%Simulating L2 Defender vs L1 Attacker - Continuos
close all
load('L2defender29.mat','L2Defender')
load('L1defender30.mat','L1Defender');
load('L1Attacker25.mat','L1Attacker');
L2Defender.L1Defender = L1Defender;
fprintf('Simulate in continuous...')
[D1A1states, D1A1defRewards, D1A1attRewards] = Simulate ( L2Defender, L0Attacker, 1, ppack );

%%
%
%Simulating L1 Defender vs L2 Attacker - Continuos
clc
close all
load('L1defender30.mat','L1Defender');
load('L2Attacker35.mat','L2Attacker');
fprintf('Simulate in continuous...')
ppack.params.nSimSteps = 200;
[D1A1states, D1A1defRewards, D1A1attRewards] = Simulate ( L1Defender, L2Attacker, 1, ppack );

%%
%To simulate N episodes and compute average performances of players, use
%Attacker.mode = params.plusminusone;
%Defender.mode = params.L0Def_mode;
%makePlot=false;
fprintf('Average reward in discrete...')
clc
clear
close all
load('L1Defender12.mat');
[defAvgReward_d, defAvgRewardSEM_d, attAvgReward_d, attAvgRewardSEM_d]... 
                = Average_rewards_d( L1Defender, L0Attacker, 50, false,ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER );

            
%%   

fprintf('Average reward in continuos...')
clc
%close all
load('L2defender26.mat','L2Defender');
load('L1Attacker25.mat','L1Attacker');
[defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM]... 
                = Average_rewards( L2Defender, L1Attacker, 50, false,ppack);
%%

fprintf('Average reward learning agents in continuos...')
clc
close all
% load('L2defender29.mat','L2Defender');
% load('L1defender30.mat','L1Defender');
% load('L1Attacker25.mat','L1Attacker');
% L2Defender.L1Defender = L1Defender;
ppack.params.nSimSteps = 200;
load('L2Attacker35','L2Attacker');
load('L1defender30','L1Defender');
[defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM]... 
                = Average_rewards( L1Defender, L2Attacker, 50, false,ppack);

%%
% Save all open plots into a single pdf, save params and BinsOrSet to text file
% and save to 'Saves' directory
SaveDescription( SaveAllOpenPlots(), ppack )