%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This matlab file provides a test example of how the 3 node   %
% NIPS model can be interacted with from within the libnfg     %
% framework                                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;

% set up a power grid system with a matlab model file
pgs = PowerGridSystem(case3);

% simulate the power grid system in its initial state
initial_pgs = pgs.simulate;

% lets inspect the voltages (the observations of the players)
% These are looking at the voltages at buses 2, and 3
% And the generation at bus 1
disp(strcat('Initial Real Generation at Bus 1: ', num2str(initial_pgs.getAttribute(1, PowerGridSystem.REAL_GENERATION))));
disp(strcat('Initial Reactive Generation at Bus 1:', num2str(initial_pgs.getAttribute(1, PowerGridSystem.REACTIVE_GENERATION))));
disp(strcat('Initial Voltage at Bus 1:', num2str(initial_pgs.getAttribute(1, PowerGridSystem.VOLTAGE_MAGNITUDE))));
disp(strcat('Initial Voltage at Bus 2:', num2str(initial_pgs.getAttribute(2, PowerGridSystem.VOLTAGE_MAGNITUDE))));
disp(strcat('Initial Voltage at Bus 3:', num2str(initial_pgs.getAttribute(3, PowerGridSystem.VOLTAGE_MAGNITUDE))));

% The players and nature adjust the "injection" at buses 2 and 3. Positive 
% injection means consumption, negative injection means production. The
% operator adjusts the voltage at bus 1

% Step 1 nature sets the P's and Q's at buses 2 and 3
next_pgs = initial_pgs.setAttribute(2, PowerGridSystem.REAL_LOAD, 0.25);
next_pgs = next_pgs.setAttribute(2, PowerGridSystem.REACTIVE_LOAD, 0.15);
next_pgs = next_pgs.setAttribute(3, PowerGridSystem.REAL_GENERATION, 0.35);
next_pgs = next_pgs.setAttribute(3, PowerGridSystem.REACTIVE_GENERATION, 0.05);

% Operator player ups the voltage at bus 1 to increase voltages at buses 2
% and 3
next_pgs = next_pgs.setAttribute(1, PowerGridSystem.VOLTAGE_MAGNITUDE, 1.02);

%Attack player transforms some of the real power into reactive, to cause a
%voltage drop - at bus 3
next_pgs = next_pgs.setAttribute(3, PowerGridSystem.REAL_GENERATION, next_pgs.getAttribute(3, PowerGridSystem.REAL_GENERATION) - 1.5);
next_pgs = next_pgs.setAttribute(3, PowerGridSystem.REACTIVE_GENERATION, next_pgs.getAttribute(3, PowerGridSystem.REACTIVE_GENERATION) + 1.5);

% simulate the outcome of the game
outcome_pgs = next_pgs.simulate;

% lets inspect the voltages of the outcome
disp(strcat('Outcome Real Generation at Bus 1: ', num2str(outcome_pgs.getAttribute(1, PowerGridSystem.REAL_GENERATION))));
disp(strcat('Outcome Reactive Generation at Bus 1:', num2str(outcome_pgs.getAttribute(1, PowerGridSystem.REACTIVE_GENERATION))));
disp(strcat('Outcome Voltage at Bus 1:', num2str(outcome_pgs.getAttribute(1, PowerGridSystem.VOLTAGE_MAGNITUDE))));
disp(strcat('Outcome Voltage at Bus 2:', num2str(outcome_pgs.getAttribute(2, PowerGridSystem.VOLTAGE_MAGNITUDE))));
disp(strcat('Outcome Voltage at Bus 3:', num2str(outcome_pgs.getAttribute(3, PowerGridSystem.VOLTAGE_MAGNITUDE))));

