function Plot_episode(trackPGS, trackMems, trackMoves, trackRewards, titleStr)
    nSteps = length(trackPGS);

    %Plot system voltages
    V1 = zeros(nSteps, 1);
    V2 = zeros(nSteps, 1);
    V3 = zeros(nSteps, 1);
    for i = 1:nSteps
        V1(i) = trackPGS{i}.getAttribute(1, PowerGridSystem.VOLTAGE_MAGNITUDE);
        V2(i) = trackPGS{i}.getAttribute(2, PowerGridSystem.VOLTAGE_MAGNITUDE);
        V3(i) = trackPGS{i}.getAttribute(3, PowerGridSystem.VOLTAGE_MAGNITUDE);
    end
    figure;
    hold on;
    plot(1:nSteps, V1, 1:nSteps, V2, 1:nSteps, V3);
    title(titleStr);
    xlabel('time (step)');
    ylabel('voltage');
    legend('V1', 'V2', 'V3');
    grid on;
    
    %Plot Memories
    defStats = zeros(nSteps, 1);
    attStats = zeros(nSteps, 1);
    for t = 1:nSteps
        defStats(t) = trackMems{t}{1}.StatD;
        attStats(t) = trackMems{t}{2}.StatA;
    end
    figure;
    subplot(2,1,1);
    plot(1:nSteps,defStats);
    title(titleStr);
    xlabel('time (step)');
    ylabel('defender''s statistic');
    grid on;
    subplot(2,1,2);
    plot(1:nSteps,attStats);
    xlabel('time (step)');
    ylabel('attacker''s statistic');
    grid on;
    
    %Plot Moves
    movedimDef = size(trackMoves{1}{1},2);
    movedimAtt = size(trackMoves{1}{2},2);
    defMoves = zeros(nSteps, movedimDef);
    attMoves = zeros(nSteps, movedimAtt);
    for t = 1:nSteps
        for d = 1:movedimDef
            defMoves(t,d) = trackMoves{t}{1}{d};
        end
        for d = 1:movedimAtt
            attMoves(t,d) = trackMoves{t}{2}{d};
        end
    end
    figure;
    subplot(2,2,1);
    plot(1:nSteps,defMoves(:,1));
    title(titleStr);
    xlabel('time (step)');
    ylabel('defender 1st move');
    grid on
    subplot(2,2,2);
    plot(1:nSteps,defMoves(:,2));
    title(titleStr);
    xlabel('time (step)');
    ylabel('defender 2nd move');
    grid on;
    subplot(2,2,3);
    plot(1:nSteps,attMoves(:,1));
    xlabel('time (step)');
    ylabel('attacker 1st move');
    grid on;
    subplot(2,2,4);
    plot(1:nSteps,attMoves(:,2));
    xlabel('time (step)');
    ylabel('attacker 2nd move');
    grid on;
    
    %Plot Rewards
    defRewards = zeros(nSteps, 1);
    attRewards = zeros(nSteps, 1);
    for t = 1:nSteps
        defRewards(t) = trackRewards{t}{1};
        attRewards(t) = trackRewards{t}{2};
    end
    figure;
    subplot(2,1,1);
    plot(1:nSteps,defRewards);
    title(titleStr);
    xlabel('time (step)');
    ylabel('defender''s reward');
    grid on;
    subplot(2,1,2);
    plot(1:nSteps,attRewards);
    xlabel('time (step)');
    ylabel('attacker''s reward');
    grid on;
end