%$$Modified by Yildiray on Sep 21st 2012
%added: 
%L0DefProps.memSize = 5;
%L0AttProps.memSize = 4;
%L0DefProps.memNumOfBinsVec = [11,8,8,11,3]; 
%L0AttProps.memNumOfBinsVec = [8,8,11,11];
%L0DefProps.CorD = 'D';
%L0AttProps.CorD = 'D';
%L0DefProps.MemBinsOrSets = BinsOrSets_MD (params);

clc
clear all
close all

starttime = tic;

params = Set_params();

% Define player memories
DefMem = Memory(@Init_MD, @Sample_MD);
AttMem = Memory(@Init_MA, @Sample_MA);
memList = [DefMem; AttMem];


% Define players
DefMoveSet = {params.dXset, [-2 0 2]};
L0DefProps.thetaD = params.thetaD;
L0DefProps.policyFcn = @Scott_Def;
L0DefProps.memNumOfElm = 6; %number of elements in the memory
L0DefProps.memNumOfBinsVec = [11,8,8,11,3,3]; %number of bins vector for memory elements
L0DefProps.CorD = 'D'; %D for discrete, C for continuous player
L0DefProps.MemBinsOrSets = BinsOrSets_MD(params); %vector of memory bin edges
L0Def = Player(@Init_Def, @Apply_Def, @Reward_Def, DefMoveSet, 3, L0DefProps);
AttMoveSet = {[-params.M:params.M], [-5 0 5]};
L0AttProps.thetaA = params.thetaA;
L0AttProps.policyFcn = @Scott_Att;
L0AttProps.memNumOfElm = 5;
L0AttProps.memNumOfBinsVec = [8,8,11,11,11];
L0AttProps.CorD = 'D';
L0AttProps.MemBinsOrSets = BinsOrSets_MA(params);
L0Att = Player(@Init_Att, @Apply_Att, @Reward_Att, AttMoveSet, 3, L0AttProps );
L0Att = L0Att.SetProbExist( params.ProbAttExist, 0 );  %Default action to 0

% Define PGS
PGS = PowerGridSystem(case3);
SIM = Simulation( @Init_PGS, params.nSimSteps, memList, params );
SIM = SIM.SetPostExecFcn( @Apply_Load );

% Initialize and run
SIM = SIM.SetPlayerList( [L0Def; L0Att] );
[trackPGS, trackMems, trackMoves, trackRewards] = SIM.Run();
Plot_episode(trackPGS, trackMems, trackMoves, trackRewards, 'D0 vs. A0');
[AvgRewardList,AvgRewardSEMList] = SIM.AverageReward(20)

% Select a learning algorithm (Jaakkola_c,FQI, etc)
Lrn_algrthm = 'Jaakkola_c';
 
%%

% Learn L1 Defender
StartLrnTime = tic;
if Lrn_algrthm == 'Jaakkola_c'
    [L1Def, info] = Train_Jaakkola_c( SIM, [L0Def; L0Att], 1 );
else if Lrn_algrthm == 'FQI'
        [L1Def, info] = Train_FQI( SIM, [L0Def; L0Att], 1 );
    end
end
EndLrnTime = toc(StartLrnTime);
datetime = int2str(datevec(now));      %current date and time
datetime(ismember(datetime,' ')) = []; %remove spaces
fnamee=[datetime '_matpower'];
save(fnamee);
% Simulate the result of the learned Defender
SIM = SIM.SetPlayerList( [L1Def; L0Att] );
[trackPGS, trackMems, trackMoves, trackRewards] = SIM.Run();
Plot_episode(trackPGS, trackMems, trackMoves, trackRewards, 'D1 vs. A0');
[AvgRewardList,AvgRewardSEMList] = SIM.AverageReward(20)

%%

% Learn L1 Attacker
StartLrnTime = tic;
if Lrn_algrthm == 'Jaakkola_c'
    [L1Att, info] = Train_Jaakkola_c( SIM, [L0Def; L0Att], 2 );
else if Lrn_algrthm == 'FQI'
        [L1Att, info] = Train_FQI( SIM, [L0Def; L0Att], 2 );
    end
end
EndLrnTime = toc(StartLrnTime);
% Simulate the result of the learned Attacker
SIM = SIM.SetPlayerList( [L0Def; L1Att] );
[trackPGS, trackMems, trackMoves, trackRewards] = SIM.Run();
Plot_episode(trackPGS, trackMems, trackMoves, trackRewards, 'D0 vs. A1');
[AvgRewardList,AvgRewardSEMList] = SIM.AverageReward(20)

%%

% Learn L2 Defender
StartLrnTime = tic;
if Lrn_algrthm == 'Jaakkola_c'
    [L2Def, info] = Train_Jaakkola_c( SIM, [L0Def; L1Att], 1 );
else if Lrn_algrthm == 'FQI'
        [L2Def, info] = Train_FQI( SIM, [L0Def; L1Att], 1 );
    end
end
EndLrnTime = toc(StartLrnTime);

% Simulate the result of the learned Defender
SIM = SIM.SetPlayerList( [L2Def; L1Att] );
[trackPGS, trackMems, trackMoves, trackRewards] = SIM.Run();
Plot_episode(trackPGS, trackMems, trackMoves, trackRewards, 'D2 vs. A1');
[AvgRewardList,AvgRewardSEMList] = SIM.AverageReward(20)

%%

% Learn L2 Attacker
StartLrnTime = tic;
if Lrn_algrthm == 'Jaakkola_c'
    [L2Att, info] = Train_Jaakkola_c( SIM, [L0Def; L1Att], 2 );
else if Lrn_algrthm == 'FQI'
        [L2Att, info] = Train_FQI( SIM, [L1Def; L0Att], 2 );
    end
end
EndLrnTime = toc(StartLrnTime);

[L2Att, info] = Train_FQI( SIM, [L1Def; L0Att], 2 );

% Simulate the result of the learned Attacker
SIM = SIM.SetPlayerList( [L1Def; L2Att] );
[trackPGS, trackMems, trackMoves, trackRewards] = SIM.Run();
Plot_episode(trackPGS, trackMems, trackMoves, trackRewards, 'D1 vs. A2');
[AvgRewardList,AvgRewardSEMList] = SIM.AverageReward(20)

endtime = toc(starttime);