function [Defender] = Train_defender_Jaakkola_c(Attacker, ppack, makeplot)
%Learn a RL Defender assuming given an Attacker
%using Jaakkola algorithm.
%This version uses epoch-wise updating of the policy.
%Input: Attacker = attacker
%Output: Defender = learned RL defender

params = ppack.params;
BinsOrSet = ppack.BinsOrSet;

fprintf('Jaakkola_defender...\n')

params.jaak = Set_params_Jaak_c();
Defender.trainParams = params.jaak;

%create a new defender
clear Defender;
Defender.mode = 4;     %Put defender into policy mode

trackIndex = 0; %for tracking

%Initializing the Jaakkola matrices and the average reward
r_av = 0;
Defender.K = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX);
Defender.beta = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX);
Defender.Q = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX);
Defender.C = ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX);

Defender.gamma_Q=params.jaak.gamma;
Defender.gamma_V=params.jaak.gamma;

%Initializing the stochastic Defender Policy Defender.pi(a|m) and greedy
%policy pi1(a|m) (see the Jaakkola paper for explanations)

Defender.pi=ones(BinsOrSet.N_V1,BinsOrSet.N_V2,BinsOrSet.N_V3,params.N_statisticD,BinsOrSet.N_dX,BinsOrSet.N_dX)/(BinsOrSet.N_dX);

%Initializing tracking matrices
Defender.deltaQ=ones(1,1000);
delta_old=1;

Defender.avreward=zeros(1,5000);
Defender.avreward_in=zeros(1,5000);

Attacker.exist=1;
%B = Sample_slice_d([], Defender, Attacker, CPTs, params, BinsOrSet);
ep=1;


Defender.pi_old=Defender.pi;
a_t_old = 2;

K_old = Defender.K; %This is used to see which states are actually observed
%by comparing old and new K values.

Defender.MeanInLoRewAv_old (1) = 0; % Running average for the defender reward
check_ind = 0;

while ep < params.jaak.nTrainEpsDef
    ep
    Defender.kl_matrix = sum(Defender.pi_old.*log(Defender.pi_old./(Defender.pi+2.2e-16)+2.2e-16),6);
    Defender.kl(ep) = mean(Defender.kl_matrix(:));
    Defender.pi_old = Defender.pi;
    
    % coin flipping on attacker's existence
    Attacker.exist = genAttExists(params.ProbAttExist);
    
    B = Sample_slice([], Defender, Attacker, ppack);
    
    InnerLoopRewAv = 0;
    for t = 1:params.jaak.NumInnerStepsDef %params.jaak.nSimSteps
        
        [S_t,OD_t,OA_t,MD_c_t,MA_t,D_t,A_t] = Slice_unpack(B);
        
        MD_t = Discretize_MD(MD_c_t, ppack);
        V1_t = MD_t.V1index;
        V2_t = MD_t.V2index;
        V3_t = MD_t.V3index;
        a_t  = Discretize_D(D_t, ppack.params);      % defenders action at time t
        SD_t = MD_t.SDindex;   % statisticsD at time t
        
        %calculate the reward
        r = Defender_reward(B, params);
        
        %Updating Jaakkola message-action matrices M(m,a) for m_t=m, a_t=a
        
        Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)=Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) + 1;
        Defender.beta=Defender.gamma_Q*Defender.beta;
        Defender.Q=Defender.Q+Defender.beta*(r-r_av);
        
        Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) = Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) - Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)*(r-r_av);
        Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)=(1-(1/Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)))*Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)+1/Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t);
        Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)=(1-(1/Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)))*Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)+Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)*(r-r_av);
        
        
        %Update the average reward estimate with the current reward
        %r_av_new = r_av + (r-r_av)/t;
        r_av_new = r_av + (r-r_av)/(params.jaak.NumInnerStepsDef*(ep-1)+t);
        InnerLoopRewAv = InnerLoopRewAv + (r-InnerLoopRewAv)/t;
        
        %Transform Jaakkola message-action matrices M(m,a) for m_t=m,
        %a_t=a using the new average reward estimate
        
        Defender.C = Defender.C + Defender.beta;
        Defender.Q = Defender.Q - Defender.C*(r_av_new-r_av);
        
        Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) = Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) + Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)*(r_av_new-r_av);
        Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) = Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) - Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t);
        
        Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) = (1-(1/Defender.K(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)))*Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)+Defender.beta(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t);
        Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t) = Defender.Q(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)-Defender.C(V1_t,V2_t,V3_t,SD_t,a_t_old,a_t)*(r_av_new-r_av);
        
        r_av = r_av_new;
        a_t_old = a_t;
        
        %Sample the next step
        B = Sample_slice(B, Defender, Attacker, ppack);
        
        %For tracking
        trackIndex = trackIndex + 1;
        
        Defender.avreward_in(t)=r_av;
        
        if(0) %Set to 1 to save full training history
            Defender.trainHistory{ep}{t} = B;
        end
    end
    
    %Update the Defender Policy Defender.pi(a|m) at the end of the episode
    %by determining the greedy polict pi1 and then updating the Defender
    %policy towards the greedy policy
    
    Defender.pi=(1-params.jaak.epsln)*Defender.pi;
    
    Q_temp = Defender.Q;
    m = max(Q_temp,[],6);
    t = bsxfun(@eq,m,Q_temp); % Identify all multiple maxima along each dim 6
    f = find(t); % Get pointers to all of them
    
    indexx = f;
    
    if (0) %make this 1 if you want to randomly pick a maximum and update it
        %0 is you want to simply update all maximums
        
        t = t*rand; %For some reason the following line does not work without this line
        t(f) = rand(length(f),1); % Multiply each 'TRUE' by a random number
        [~,ind] = max(t,[],6); % Find the maximums of these random numbers
        [I,J,K,P,R] = ind2sub(size(ind),1:size(ind(:)));
        
        indexx = sub2ind(size(Defender.pi),I,J,K,P,R,ind(:)');
    end
    
    Defender.pi(indexx) = ...
        Defender.pi(indexx) + params.jaak.epsln*1;
    
    normz = repmat(sum(Defender.pi,6), [1, 1, 1, 1, 1, BinsOrSet.N_dX]);
    Defender.pi = Defender.pi ./ normz;
    
    %For tracking
    Defender.avreward(ep) = r_av;
    Defender.InnerLoopRewAv(ep) = InnerLoopRewAv;
    
    %     sumkl = sum(Defender.pi_old.*log(Defender.pi_old./Defender.pi),6);
    %     sument = -sum(Defender.pi.*log2(Defender.pi),6);

    if mod(ep,40) == 1
        check_ind = check_ind + 1;
        fprintf('Average reward in continuous...')
        [defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM, ...
            defAvgNumMove, defAvgNumMoveSEM, AvgSocWelf, AvgSocWelfSEM] ...
            = Average_rewards( Defender, Attacker, 100, false, ppack);
        
        Defender.defAvgReward_RealCheck(check_ind) = defAvgReward;
        Defender.defAvgRewardSEM_RealCheck(check_ind) = defAvgRewardSEM;
    end
    
    if mod(ep,40) == 1
        check_ind = check_ind + 1;
        fprintf('Average reward in continuous...')
        [defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM, ...
            defAvgNumMove, defAvgNumMoveSEM, AvgSocWelf, AvgSocWelfSEM] ...
            = Average_rewards( Defender, Attacker, 100, false, ppack);
        
        Defender.defAvgReward_RealCheck(check_ind) = defAvgReward;
        Defender.defAvgRewardSEM_RealCheck(check_ind) = defAvgRewardSEM;
    end
    
    ep=ep+1;
end


if makeplot
    figure
    plot(Defender.InnerLoopRewAv);
    title('Running window average reward');
    
    figure
    plot(Defender.avreward);
    title('Average reward');
    
    figure
    plot(Defender.defAvgReward_RealCheck);
    title('Real Average reward');
    
    figure
    errorbar(Defender.defAvgReward_RealCheck,Defender.defAvgRewardSEM_RealCheck);
    title('Real Average reward with SEM');
    
    
end


end
