%Class for players

%$$Modified by Yildiray Yildiz on 09/19/12
%added 'Get memory size' function
%added: if P.props.CorD == 'D' ... (see the rest in the code)
%added: "% Calculate the bin numbers of the memory" method
%added: elseif (P.mode == 'Jaakkola_c')

%$$Modified by Yildiray Yildiz on 09/27/12
%added "moveDim" for "move dimension" to the properties
                     

classdef Player
    properties (SetAccess = private)
      mode;
      props;
      initFcn;
      applyFcn;
      rewardFcn;
      moveSet;
      policyFcn;
      probExist;
      notExistAction;
      playExist;
      moveDim;
    end % properties
    
   methods
        % Constructor
        function P = Player( initFcn, applyFcn, rewardFcn, moveSet, inmode, inprops )
            
            if ( exist('inmode','var') && exist('inprops','var') )
                P = SetMode(P, inmode, inprops );
            end
            
            P.initFcn = initFcn;
            P.applyFcn = applyFcn;
            P.rewardFcn = rewardFcn;
            P.moveSet = moveSet;
            P.probExist = 1.0;
            P.notExistAction = [];
            P.playExist = 1;
            P.moveDim = size(moveSet,2);
        end % Constructor
        
        % set mode and properties
        function P = SetMode(P, inmode, inprops)
            P.mode = inmode;
            P.props = inprops;
        end
        
        % set probability of existence
        function P = SetProbExist(P, inprob, notExistAction)
            P.probExist = inprob;
            P.notExistAction = notExistAction;
        end
        
        % Get move set
        function moveSet = GetMoveSet( P )
            moveSet = P.moveSet;
        end
        
        % Get properties
        function props = GetProperties( P )
            props = P.props;
        end
        
        % Initial move
        function [P,A] = Init(P, params)
            r = rand();
            if ( (r > P.probExist) && ~isempty(P.notExistAction) )
                P.playExist = 0;
                for i=1:P.moveDim
                    A{i} = P.notExistAction;
                end
            else
                P.playExist = 1;
                A = feval( P.initFcn, params );
            end
        end
        
        % Play a move
        function A = Play(P, Mem, params)
            % Randomize for probability of existence
            if ( ~P.playExist )
                A = P.notExistAction;
            else
                %Pick an action based on memory M
                %If the player is discrete, first find bin index of each
                %memory
                if P.props.CorD == 'D'
                    Mem.vec = P.GetMemBinInd(Mem);
                end                 
                if ( P.mode == 1 ) %fixed
                    A = P.props.fixedmove;
                elseif ( P.mode == 2 ) %uniformly random
                    A = P.moveSet( randi(length(P.moveSet)) );
                elseif ( P.mode == 3 ) %custom
                    A = feval( P.props.policyFcn, P, Mem, params );
                    for i=1:P.moveDim
                        ind(i) = find( P.moveSet{i} == A{i}, 1 );
                        assert( ~isempty(ind(i)) ); %make sure the move is in the moveset
                    end                
                elseif ( P.mode == 5 ) %linear FQI
                    [~,ai] = FitQLin_GetQmax( P.props.QWts, Mem.vec, P.moveSet);
                    A = P.moveSet(ai);
                elseif ( P.mode == 6 ) %linear FQI
                    [~,ai] = FitQNN_GetQmax( P.props.net, Mem.vec, P.moveSet);
                    A = P.moveSet(ai);
                elseif (P.mode == 'Jaakkola_c')
                    Mem.vec_ind = num2cell(Mem.vec);
                    ai = Weighted_sample(squeeze(P.props.pi(Mem.vec_ind{:},:,:)));
                    for i=1:P.moveDim
                        A{i} = P.moveSet{i}(ai{i});
                        ind(i) = find( P.moveSet{i} == A{i}, 1 );
                        assert( ~isempty(ind(i)) ); %make sure the move is in the moveset
                    end
                else
                    assert(false);
                end
            end
        end
        
        % Apply move
        function PGS = ApplyMove(P, PGS, A, params)
            PGS = feval( P.applyFcn, PGS, A, params );
        end
        
        % Reward function
        function r = GetReward( P, PGS, mems, moves, params )
            r = feval( P.rewardFcn, PGS, mems, moves, params );
        end
        
        % Get memory size
        function memNumOfElm = GetMemNumOfElm(P)
            memNumOfElm = P.props.memNumOfElm;
        end
        
        % Get number of bins vector for the memory
        function memNumOfBinsVec = GetMemNumOfBinsVec(P)
            memNumOfBinsVec = P.props.memNumOfBinsVec;
        end
        
        % Calculate the bin index numbers of the memory
        function BinInd = GetMemBinInd(P,Mem)
            for i=1:numel(Mem.vec)
                [~,BinInd(i)] = histc(Mem.vec(i), P.props.MemBinsOrSets.binsVec{i}); 
            end
        end
        
        
   end % methods 
end % classdef
      

