
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20120913	Created
//
// *************************************************************************

#include "cpd_adder.h"

using namespace nfg;

// *************************************************************************
// *************************************************************************
cpd_Adder::cpd_Adder( const double addVal )
{	
	m_addVal = addVal;
}
// *************************************************************************
// *************************************************************************
cpd_Adder::~cpd_Adder ( void )
{
}
// *************************************************************************
// *************************************************************************
nfg_Vec cpd_Adder::Sample_node ( const nfg_Vecs& paVecs )
{
	//	make sure only 1 parent, and that parent vec size is 1 
	bassert ( paVecs.size() == 1 && paVecs.at(0).size() == 1);

	return nfg_Vec( 1, paVecs.at(0).at(0) + m_addVal );
}
// *************************************************************************
// *************************************************************************
double cpd_Adder::Get_probability( const nfg_Vec& outVec, const nfg_Vecs& paVecs )
{
	//	make sure only 1 parent, and that parent vec size is 1 
	bassert ( paVecs.size() == 1 && paVecs.at(0).size() == 1);

	//	make sure output is a real
	bassert ( outVec.size() == 1 );

	return (outVec.at(0) == (paVecs.at(0).at(0) + m_addVal)) ? 1.0 : 0.0;
}
// *************************************************************************
// *************************************************************************
void cpd_Adder::Unit_test ( void )
{
	//	Set up variance vector
	cpd_Adder testAdd( 0.2 );

	//	Set up parents vector list
	nfg_Vecs paVecs(1, nfg_Vec(1, 1.0));

	//	Test sample_node()
	nfg_Vec outVec = testAdd.Sample_node( paVecs );
	Print_vec(outVec);

	//	Test Get_prob()
	cout << "Prob: " << testAdd.Get_probability_prop( outVec,  paVecs ) << endl;

	outVec.at(0) = 0.0;
	cout << "Prob: " << testAdd.Get_probability_prop( outVec,  paVecs ) << endl;

	cout << "Unit test completed." << endl << endl;
}

