
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20110526	Created
//	rlee		20110816	Converted to inherited class
//
// *************************************************************************

#ifndef INCLUDED_CPD_BERNOULLI_H
#define INCLUDED_CPD_BERNOULLI_H

#include "types.h"
#include "helper.h"
#include "cpd.h"

/**
 * cpd_Bernoulli: CPD class that implements Bernoulli distributions in a vector.
 *		Each component is a Bernoulli random variable parameterized by the corresponding
 *		vector component in probSuccs in the constructor.
 *		Expects no parent vectors.  i.e., acts as a prior.
 */
class cpd_Bernoulli : public cpd
{
public:

	/**	Constructor */
	cpd_Bernoulli( const nfg_Vec& probSuccs );

	/**	Destructor */
	virtual ~ cpd_Bernoulli();

	/// Variables

	///	Methods
	/**
	 * Sample_node: Sample the node given parents
	 *	paVecs - parent vectors
	 *	returns: sample value of node
	 */
	nfg_Vec	Sample_node ( const nfg_Vecs& paVecs );
	/**
	 * Get_probability: Get probability of a vector given parents
	 *	outVec - a sample/realization of the node
	 *	paVecs - parent vectors
	 *	returns: probability that this occurred
	 */
	double	Get_probability ( const nfg_Vec& outVec, const nfg_Vecs& paVecs );

	// Debug
	/**
	 * Unit_test: test to verify that this cpd works correctly.
	 */
	static void Unit_test ( void );

private:

	/// Variables
	nfg_Vec			m_probs;		// p's, aka prob of success is the main 
									// parameter in bernoulli distribution, one for each dimension

	/// Methods

};

#endif

