
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20120914	Created
//
// *************************************************************************

#include "cpd_glue.h"

using namespace nfg;

// *************************************************************************
// *************************************************************************
cpd_Glue::cpd_Glue( void )
{	
}
// *************************************************************************
// *************************************************************************
cpd_Glue::~cpd_Glue ( void )
{
}
// *************************************************************************
// *************************************************************************
nfg_Vec cpd_Glue::Sample_node ( const nfg_Vecs& paVecs )
{
	//	make sure only 1 parent
	bassert ( paVecs.size() == 1 );

	return paVecs.at(0);
}
// *************************************************************************
// *************************************************************************
double cpd_Glue::Get_probability( const nfg_Vec& outVec, const nfg_Vecs& paVecs )
{
	//	make sure only 1 parent
	bassert ( paVecs.size() == 1 );

	//	make sure output is the same size as parent
	bassert ( outVec.size() == paVecs.at(0).size() );

	bool match = true;

	for( unsigned int i = 0; i < outVec.size(); i++ )
		match &= ( outVec.at(i) == paVecs.at(0).at(i) );

	return match ? 1:0;
}
// *************************************************************************
// *************************************************************************
void cpd_Glue::Unit_test ( void )
{
	//	Set up variance vector
	cpd_Glue gl;

	//	Set up parents vector list
	nfg_Vecs paVecs(1, nfg_Vec(2, 1.2));

	//	Test sample_node()
	nfg_Vec outVec = gl.Sample_node( paVecs );
	Print_vec(outVec);

	//	Test Get_prob()
	cout << "Prob: " << gl.Get_probability_prop( outVec,  paVecs ) << endl;

	outVec.at(0) = 0.0;
	cout << "Prob: " << gl.Get_probability_prop( outVec,  paVecs ) << endl;

	cout << "Unit test completed." << endl << endl;
}

