
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20101025	Created
//	rlee		20110816	Converted to inherited class
//
// *************************************************************************

#ifndef INCLUDED_CPD_TABLEBASIC_H
#define INCLUDED_CPD_TABLEBASIC_H

#include <boost/math/special_functions/round.hpp>
#include "ndarray.h"
#include "types.h"
#include "helper.h"
#include "cpd.h"

/**
 * cpd_TableBasic: Implements the CPD of a discrete node.  Uses a probability table.
 * TableBasic assumes vector size 1 on its output and all its parents.
 */
class cpd_TableBasic : public cpd
{
public:

	/**	Constructor */
	/**
	 * cpd_TableBasic: Constructor
	 *	dims - a vector of sizes. dims = [[sizeof(out)] [sizeof(pa0)] [sizeof(pa1)] ...]
	 *	vals - specifies the values of the CPD table.
	 *		See unit_test for organization.
	 */
	cpd_TableBasic( const nfg_Dim& dims, const nfg_Vec& vals );

	/**	Destructor */
	virtual ~ cpd_TableBasic();

	/// Variables

	///	Methods
	/**
	 * Sample_node: Sample the node given parents
	 *	paVecs - parent vectors
	 *	returns: sample value of node
	 */
	nfg_Vec	Sample_node ( const nfg_Vecs& paVecs );
	/**
	 * Get_probability: Get probability of a vector given parents
	 *	outVec - a sample/realization of the node
	 *	paVecs - parent vectors
	 *	returns: probability that this occurred
	 */
	double	Get_probability ( const nfg_Vec& outVec, const nfg_Vecs& paVecs );

	// Debug
	static void Unit_test ( void );

private:

	/// Variables
	NDArray		m_table;		// Use a multi-dimensional array to store the probability table

	/// Methods	
};

#endif

