
// *************************************************************************
//	Net-form game library
//
//	Author:		Ritchie Lee
//
//	The contents of this file is part of the Net-form game library.
//	Copyright 2010 by NASA Ames Research Center. All rights reserved.
//	Do not duplicate, publish, or use the contents of this
//	file in any way without express written consent from
//	NASA Ames Research Center, or in accordance with
//	the terms stipulated in the license agreement.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20110324	Created
//
// *************************************************************************

#include "if_matlab.h"
#include <string>
#include <iostream>
#include <fstream>
#include <boost/algorithm/string.hpp>

using namespace std;

// *************************************************************************
// *************************************************************************
if_Matlab::if_Matlab( void )
{
}
// *************************************************************************
// *************************************************************************
if_Matlab::~if_Matlab ( void )
{
}
// *************************************************************************
// *************************************************************************
void if_Matlab::Export_WtdInsts_to_file( const string filename, 
												  const string varName,
												const nfg_WtdInsts& wInsts,
												bool append )
{
	ofstream mfile;

	//	Open file to write
	if ( append )
	{
		mfile.open( filename.c_str(), ios_base::app );
		mfile << endl;
	}
	else
		mfile.open( filename.c_str() );

	if (mfile.is_open())
	{
		cout << "The file is open";
	}
	else
	{
	    cout << " The file could not be opened";
	}


	int inst_i = 1;	//	Matlab counting starts at 1
	foreach ( nfg_WtdInst wInst, wInsts )
	{
		//	declare a cell array {instantiation index, vertex index}
		bassert( wInst.m_weight == wInst.m_weight);	// Check that weight is finite
		
		mfile << varName << "{" << inst_i << ",1}=" << wInst.m_weight << ";" << endl;
		int vi = 1;	//	Matlab counting starts at 1
		foreach ( nfg_Vec vec, wInst.m_inst )
		{
			//	Offset 1 to account for weight
			mfile << varName << "{" << inst_i << "," << vi+1 << "}=[";

			foreach( double val, vec )
			{
				mfile << val << ",";
			}

			mfile << "];" << endl;
			vi++;
		}
		inst_i++;
	}

	mfile.close();

	return;
}
// *************************************************************************
// *************************************************************************
void if_Matlab::Export_Vec_to_file( const string filename, const string varName,
												const nfg_Vec& dblVec, bool append )
{
	ofstream mfile;

	//	Open file to write
	if ( append )
	{
		mfile.open( filename.c_str(), ios_base::app );
		mfile << endl;
	}
	else
		mfile.open( filename.c_str() );

	mfile << varName << "=[";
	foreach ( double d, dblVec )
	{
		bassert( d==d );	// check that the number is finite
		mfile << d << ";";
	}
	mfile << "];";

	mfile.close();

	return;
}
// *************************************************************************
// *************************************************************************
string if_Matlab::Replace_Matlab_special_chars ( const string& inStr )
{
	string str(inStr);
	
	//	Replace underscores with spaces if they exist
	replace_all(str, "_", " ");
	//	Replace apostrophe with double apostrophe
	replace_all(str, "\'", "\'\'");

	return str;

}

