
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	yildiray	20111123	Created
//
// *************************************************************************

#ifndef INCLUDED_NET_BATTLEOFSEXES_H
#define INCLUDED_NET_BATTLEOFSEXES_H

#include "helper.h"
#include "types.h"
#include "bayesnet.h"

/**
 * "Battle of Sexes" example. See wikipedia page.
 *	Game played off a payoff matrix.  Players pick between Opera and Football.
 *	Different combinations give different payoffs.  There are two pure Nash equilibriums,
 *  (Opera,Opera) and (Football, Football). Different from the "Battle of Sexes" game,
 *  additional harm is accounted from not only going to different games but also going
 *  to the wrong game (e.g. Man going to the Opera and Woman going to the Football Game).
 *	Demonstrates normal form game with Nash Equilibrium.
 */
class net_BattleOfSexes
{
	// Make convenient labels for the vertices
	enum { D1, D2, N };

public:
	/**
	 * Run: Execute example sequence
	 */
	static void			Run ( void );
	/** 
	 * Util_function_P1: P1's utility function
	 *	instVecs: an instantiation of the net
	 *	returns: utility value
	 */
	static double		Util_function_P1 ( const nfg_Inst& instVecs );
		/** 
	 * Util_function_P2: P2's utility function
	 *	instVecs: an instantiation of the net
	 *	returns: utility value
	 */
	static double		Util_function_P2 ( const nfg_Inst& instVecs );
	/**
	 * Sample_movespace: Sample a possible move of a decision node 
	 *	(satisficing distribution)
	 *	paVecs: a list of parent vectors
	 *	returns: a realization of the node
	 */
	static nfg_Vec		Sample_movespace ( const nfg_Vecs& paVecs );
	/**
	 * Sample_level0: Sample the level 0 distribution of a decision node 
	 *	(level 0 distribution)
	 *	paVecs: a list of parent vectors
	 *	returns: a realization of the node
	 */
	static nfg_Vec		Sample_level0 ( const nfg_Vecs& paVecs );
	/**
	 * Analysis1: Performs a series of analyses based on a bnet
	 *	bnet: nfg object
	 */
	static void			Analysis1 ( BayesNet& bnet );
	static void			Analysis2 ( BayesNet& bnet );
};

#endif

