
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20110328	Created
//
// *************************************************************************

#ifndef INCLUDED_NET_BOBVSNATURE_H
#define INCLUDED_NET_BOBVSNATURE_H

#include "helper.h"
#include "types.h"
#include "bayesnet.h"

/**
 * "Bob vs nature": Example 1 from Game Theoretic Modeling of Pilot
 *	Behavior during Mid-Air Encounters (Lee & Wolpert)
 *	No edges. Bob and nature both pick from 0 to 5.  Bob's utility function
 *	is the difference between the two.  Nature is a chance node.
 *	Demonstrates difference between relaxed and d-relaxed strategies
 */
class net_BobVsNature
{
	// Make convenient labels for the vertices
	enum { D, C, N };

public:
	/**
	 * Run: Execute example sequence
	 */
	static void			Run ( void );
	/** 
	 * Util_function_P1: P1's utility function
	 *	instVecs: an instantiation of the net
	 *	returns: utility value
	 */
	static double		Util_function_P1 ( const nfg_Inst& instVecs );
	/**
	 * Sample_movespace_D: Sample a possible move of a decision node 
	 *	(satisficing distribution of node D)
	 *	paVecs: a list of parent vectors
	 *	returns: a realization of the node
	 */
	static nfg_Vec		Sample_movespace_D ( const nfg_Vecs& paVecs );
	/**
	 * Sample_level0_D: Sample the level 0 distribution of a decision node 
	 *	(level 0 distribution of node D)
	 *	paVecs: a list of parent vectors
	 *	returns: a realization of the node
	 */
	static nfg_Vec		Sample_level0_D ( const nfg_Vecs& paVecs );
	/**
	 * Analysis1: Performs a series of analyses based on a bnet
	 *	bnet: nfg object
	 */
	static void			Analysis1 ( BayesNet& bnet );
};

#endif

