
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20110328	Created
//
// *************************************************************************

#include "player.h"

#include <string>
#include <algorithm>
#include <iostream>

using namespace nfg;

// *************************************************************************
// *************************************************************************
Player::Player( const nfg_UtilFunc& fo_Get_util, const string name )
{
	// Set utility function
	m_fo_Get_util			= fo_Get_util;

	// Set player name
	m_name					= name;
	
	// empty controlled node list
	m_controlledNodes.clear();
}
// *************************************************************************
// *************************************************************************
Player::~Player ( void )
{
}
// *************************************************************************
// *************************************************************************
nfg_UtilFunc&	Player::Get_fo_of_Get_util ( void )
{
	// function object to utility function
	return m_fo_Get_util;
}
// *************************************************************************
// *************************************************************************
void Player::Add_controlled_node ( const int nodeId )
{
	// if it isn't in the list, add it
	if ( !Is_controlled(nodeId) )
	{
		m_controlledNodes.push_back(nodeId);
		m_controlledNodes.sort();
	}
	// if it's already there, don't add
}
// *************************************************************************
// *************************************************************************
bool Player::Is_controlled ( const int nodeId )
{
	// Check if it's in the list
	// Makes use of std algorithm function
	return ( (int)std::count( m_controlledNodes.begin(), m_controlledNodes.end(), nodeId ) > 0 );
}
// *************************************************************************
// *************************************************************************
void Player::Unit_test ( void )
{
	// Test various controlled node list functionality

	// Declare a player
	Player P1 ( NULL );
	
	cout << "Adding Node1..." << endl;
	P1.Add_controlled_node ( 1 );	// add a node to the list
	cout << "Num nodes: " << P1.Get_num_controlled_nodes() << endl;	// show number in list
	
	cout << "Adding Node2..." << endl;
	P1.Add_controlled_node ( 2 );	// add a node to the list
	cout << "Num nodes: " << P1.Get_num_controlled_nodes() << endl;	// show number in list
	
	cout << "Adding Node1..." << endl;
	P1.Add_controlled_node ( 1 );	// add a node to the list
	cout << "Num nodes: " << P1.Get_num_controlled_nodes() << endl;	// show number in list
	
	// Test is_controlled
	cout << "Node1 is controlled: " << (P1.Is_controlled( 1 ) ? "true":"false");
	cout << endl;
	cout << "Node3 is controlled: " << (P1.Is_controlled( 3 ) ? "true":"false");
	cout << endl;

	cout << "Clearing list..." << endl;
	P1.Clear_controlled_nodes();		// Clear the list
	cout << "Num nodes: " << P1.Get_num_controlled_nodes() << endl;	// show number in list
}

