# -*- coding: utf-8 -*-
"""
Created on Tue Oct 30 13:21:29 2012

@author: jamesbono

A proof of concept script creating discrete and continuous chance node objects
for the purpose of converting NFGs into PyMC models after supplying CPTs to 
decision nodes in an outer loop
"""

import pymc as pm
import numpy as np
from types import *

class Cont_Chance_Node(object):
    """
    a mockup of an NFG Continuous chance node with the following attributes
    ---------
    Arguments
    ---------
    pymc_obj:       (PyMC Stochastic) enter a pymc object from a continuous 
                    distribution that has parents specified as the pymc attributes
                    of other NFG node objects
    description:    (string) a verbal description of the chance node, e.g. 
                    'There are two possible values, 0 and 1, where 0 occurs 
                    with probability p and 1 occurs with probability 1-p. Here 
                    p is given by the parent node which is Uniform(0,1)'. Note
                    that because there is no space attribute for 
                    Cont_Chance_Node, this is a good place to describe the 
                    underlying space at this node
    -----------------------
    Computed from Arguments
    -----------------------
    name:           (string) name of the chance node, assigned to 
                    pymc_obj.__name__
    
    """
    def __init__( self, parents, pymc_obj, description=""):
        # Checking types of the inputs
        self.pymc = pymc_obj
        # name is a string
        # assert type(name) is StringType, "name is not a string, it is a %r" % type(name)             
        self.name = self.pymc.__name__
        # description is a string
        assert type(description) is StringType, \
            "description is not a string, it is a %r" % type(description)
        self.description = description
        # creating a parents attribute ordered dictionary from self.pymc.parents
        parent_dict = OrderedDict( sorted(self.pymc.parents.items(), \
            key=lambda t: t[0] ) )
        self.parents = OrderedDict()
        # populating the OrderedDict with NFG parents and keys
        for par in parent_dict.keys():
            """the eval statement 'should' grab the NFG object that corresponds
            to the string par. This means the names in pymc_obj.parents must correspond
            to the actual NFG parents"""
            self.parents[par] = eval(par)
    
    def __str__( self ):
        """
        Getting the chance node to reveal basic information about itself.
        """
        return "A continuous chance node called "+self.name

class Disc_Chance_Node(object):
    """
    a mockup of an NFG Discrete chance node with the following attributes
    ---------
    Arguments
    ---------
    pymc_obj:       (PyMC Stochastic) enter a pymc object from a discrete 
                    distribution that has parents specified as the pymc attributes
                    of other NFG node objects
    description:    (string) a verbal description of the chance node, e.g. 
                    'There are two possible values, 0 and 1, where 0 occurs 
                    with probability p and 1 occurs with probability 1-p. Here 
                    p is given by the parent node which is Uniform(0,1)'. Note
                    that because there is no space attribute for 
                    Cont_Chance_Node, this is a good place to describe the 
                    underlying space at this node
    -----------------------
    Computed from Arguments
    -----------------------
    name:           (string) name of the chance node, assigned to 
                    pymc_obj.__name__
    
    """