# -*- coding: utf-8 -*-
"""
Created on Wed Nov 14 10:15:43 2012

@author: Yildiray Yildiz, 

A proof of concept script creating a stochastic chance node 
object for the purpose of converting NFGs into PyMC models after supplying
the distribution type (Uniform, Gaussian etc). These types can be chosen
directly from pymc built-in distributions using "builtin_dist" or a hand 
made "my_dist" can be supplied.

The paramaters of the distributions consists of two parts. First part is an nfg
object, the second part, "bias_*" is a float. For example, for the "lower" parameter used
in the "DiscreteUniform" distribution, the first part, "lower" is an nfg object
and second part, "bias_lower" is a float. When creating the stochastic object, "lower"
parameter is defined as: "lower=lower+bias_lower". This way, the user can use an
nfg object as a distribution parameter, or she can use any floating point number, or
seh can use both. 
"""

from types import *
import pymc as pm

class StcChanceNode(object):
    def __init__(self, name, space=[], description="", builtin_dist="", 
                  lower=None, bias_lower=0.0, upper=None, bias_upper=0.0, 
                  beta=None, bias_beta=0.0, mu=None, bias_mu=0.0, tau=None, 
                  bias_tau=0.0, my_dist=""):
        # name is a string
        assert type(name) is StringType, "name is not a string it is a %r" \
            %type(name)
        self.name=name
        # space is a list
        assert type(space) is ListType, "space is not a list it is a %r" \
            %type(space)
        self.space=space
        # description is a string
        assert type(description) is StringType, "description is not a " \ 
            "string, it is a %r" % type(description)
        # builtin_dist is a string
        assert type(builtin_dist) is StringType, "builtin_dist is not a " \ 
            "string it is a %r" %type(builtin_dist)
        self.builtin_dist=builtin_dist  
        # my_dist is a string
        assert type(my_dist) is StringType, "my_dist is not a string it is " \
            "a %r" %type(builtin_dist)
        self.my_dist=my_dist
        # lower, upper, beta and mu should be nfg objects, if they are to be used in
        # the object creation. They are then stored in temp variables later to be
        # used in the convert_to_pymc method.
        self.temp_lower=lower
        self.temp_upper=upper
        self.temp_beta=beta
        self.temp_mu=mu   
        # parent pymc objects are assigned to child (self) nfg attributes as strings because 
        # parent pymc objects are not created yet during nfg object creation. These strings will
        # be "evaluated" in the convert_to_pymc method below because by that time parent pymc
        # objects will be available. If any distribution parameter is not defined, i.e.
        # set to default value "None", then they are set to 0.0
        if lower!=None:
            self.lower="self.temp_lower.pymc"
        else: self.lower="0.0"
        if upper!=None:
            self.upper="self.temp_upper.pymc"
        else: self.upper="0.0"
        if beta!=None:
            self.beta="self.temp_beta.pymc"
        else: self.beta="0.0"
        if mu!=None:
            self.mu="self.temp_mu.pymc"
        else: self.mu="0.0"
        if tau!=None:
            self.tau="self.temp_tau.pymc"
        else: self.tau="0.0"
        #bias_lower, bias_upper, bias_mu, bias_tau are floats
        assert type(bias_lower) is FloatType and \
               type(bias_upper) is FloatType and \
               type(bias_beta)  is FloatType and \
               type(bias_mu)    is FloatType and \
               type(bias_tau)   is FloatType,    \
               "builtin_dist parameters must be strings"
        self.bias_lower=bias_lower
        self.bias_upper=bias_upper
        self.bias_beta=bias_beta
        self.bias_mu=bias_mu
        self.bias_tau=bias_tau
    
    def convert_to_pymc(self):
        if self.my_dist=="":
            if self.builtin_dist=="DiscreteUniform": 
                self.pymc=pm.DiscreteUniform(str(self.name), lower=eval(self.lower)+self.bias_lower, 
                    upper=eval(self.upper)+self.bias_upper)
            elif self.builtin_dist=="Normal":
                self.pymc = pm.Normal(str(self.name), mu=eval(self.mu)+self.bias_mu, 
                    tau=eval(self.tau)+self.bias_tau)
            elif self.builtin_dist=="Poisson":
                self.pymc = pm.Poisson(str(self.name), mu=eval(self.mu)+self.bias_mu)
            
        
